#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""
from typing import Any, List, Union, Optional, Tuple
import numpy as np
import datetime

from PySide2.QtWidgets import (
    QGraphicsPolygonItem,
    QGraphicsEllipseItem,
    QGraphicsTextItem
)
from PySide2.QtGui import QBrush, QPolygon, QFont, QFontMetrics, QPen
from PySide2.QtCore import QPoint
import scine_database as db
import scine_utilities as utils

from scine_heron.utilities import qcolor_by_key


class Structure(QGraphicsEllipseItem):
    def __init__(self, x, y, brush=None, pen=None) -> None:
        # The ellipse is displayed based on its top left corner.
        #  With a size of 20x20 units a shift of -10,-10 centers it.r = 20.0
        r = 20.0
        shift = 10.0

        super().__init__(x - shift, y - shift, r, r)
        self.x_coord = x
        self.y_coord = y
        self.db_representation: Any = None
        self.__brush = brush
        self.reset_brush()
        if pen is not None:
            self.setPen(pen)
        self._mouse_release_function = None
        self._mouse_double_click_function = None
        self._mouse_press_function = None
        self._hover_enter_function = None
        self._hover_leave_function = None
        self._menu_function = None

    def reset_brush(self):
        if self.__brush is not None:
            self.setBrush(self.__brush)

    def set_brush(self, brush: QBrush):
        self.__brush = brush
        self.setBrush(brush)

    def center(self):
        return QPoint(self.x_coord, self.y_coord)

    def bind_mouse_release_function(self, func):
        self._mouse_release_function = func

    def bind_mouse_press_function(self, func):
        self._mouse_press_function = func

    def bind_mouse_double_click_function(self, func):
        self._mouse_double_click_function = func

    def bind_hover_enter_function(self, func):
        self._hover_enter_function = func

    def bind_hover_leave_function(self, func):
        self._hover_leave_function = func

    def bind_menu_function(self, func):
        self._menu_function = func

    def mouseDoubleClickEvent(self, event):
        if self._mouse_double_click_function is not None:
            self._mouse_double_click_function(event, self)

    def mousePressEvent(self, event):
        if self._mouse_press_function is not None:
            self._mouse_press_function(event, self)

    def mouseReleaseEvent(self, event):
        if self._mouse_release_function is not None:
            self._mouse_release_function(event, self)

    def hoverEnterEvent(self, event):
        if self._hover_enter_function is not None:
            self._hover_enter_function(event, self)

    def hoverLeaveEvent(self, event):
        if self._hover_leave_function is not None:
            self._hover_leave_function(event, self)

    def contextMenuEvent(self, event):
        if self._menu_function is not None:
            self._menu_function(event, self)


class Compound(QGraphicsEllipseItem):
    def __init__(self, x, y, brush=None, pen=None,
                 concentration: Union[float, None] = None, cost: Union[float, None] = None) -> None:
        # The ellipse is displayed based on its top left corner.
        #  With a size of 20x20 units a shift of -10,-10 centers it.
        self.r = 20.0
        self.shift = 10.0
        self.concentration: Union[float, None] = concentration
        self.final_concentration: Union[float, None] = None
        self.concentration_flux: Union[float, None] = None
        self.cost: Union[float, None] = cost

        self.allow_scaling = False

        self.x_coord = x
        self.y_coord = y
        super().__init__(self.x_coord - self.shift,
                         self.y_coord - self.shift,
                         self.r, self.r)
        self.created: Union[datetime.datetime, None] = None
        self.db_representation: Any = None
        self.__brush = brush
        self.__current_brush = brush
        self.__pen = pen
        self.__current_pen = pen

        self.reset_brush()
        if pen is not None:
            self.setPen(pen)
        self._mouse_release_function = None
        self._mouse_double_click_function = None
        self._mouse_press_function = None
        self._hover_enter_function = None
        self._hover_leave_function = None
        self._menu_function = None

    def update_size(self):
        scaling = 1.0 if not self.allow_scaling else self.get_scaling()
        tmp_r = self.r * scaling
        tmp_shift = self.shift * scaling
        self.setRect(self.x_coord - tmp_shift, self.y_coord - tmp_shift, tmp_r, tmp_r)

    def get_scaling(self):
        from math import log10
        scaling = 1.0
        if self.concentration is not None:
            concentration = max(self.concentration, 1e-9)
            # Scale the size of the compound according to its concentration.
            # Minimum scaling: 10 % of the original radius.
            # Note that the scaling function is more or less made up and can be changed if required.
            scaling = 1.5 * max(1.0 / (1.0 + 0.5 * abs(log10(concentration))), 0.1)
        elif self.cost is not None:
            cost = min(self.cost, 100)
            if cost == 0.0:
                scaling = 0.6
            else:
                scaling = 5.0 + cost * -0.025
        return scaling

    def add_concentration_tooltip(self):
        cs = [self.concentration, self.final_concentration, self.concentration_flux]
        names = ["c_max", "c_final ", "c_flux "]
        self.setToolTip(f"{names[0]:6} = {cs[0]:.3E}\n"
                        f"{names[1]:8} = {cs[1]:.3E}\n"
                        f"{names[2]:8} = {cs[2]:.3E}")

    def reset_brush(self):
        if self.__brush is not None:
            self.__current_brush = self.__brush
            self.setBrush(self.__brush)
            self.__current_pen = self.__pen
            self.setPen(self.__pen)

    def set_current_brush(self, brush: QBrush):
        self.__current_brush = brush
        self.setBrush(brush)

    def get_current_brush(self):
        return self.__current_brush

    def get_brush(self) -> Union[None, QBrush]:
        return self.__brush

    def get_pen(self) -> Union[None, QPen]:
        return self.__pen

    def set_current_pen(self, pen: QPen):
        self.__current_pen = pen
        self.setPen(pen)

    def get_current_pen(self):
        return self.__current_pen

    def center(self):
        return QPoint(self.x_coord, self.y_coord)

    def set_created(self, created: datetime.datetime):
        self.created = created

    def get_created(self) -> Union[None, datetime.datetime]:
        return self.created

    def bind_mouse_release_function(self, func):
        self._mouse_release_function = func

    def bind_mouse_press_function(self, func):
        self._mouse_press_function = func

    def bind_mouse_double_click_function(self, func):
        self._mouse_double_click_function = func

    def bind_hover_enter_function(self, func):
        self._hover_enter_function = func

    def bind_hover_leave_function(self, func):
        self._hover_leave_function = func

    def bind_menu_function(self, func):
        self._menu_function = func

    def mouseDoubleClickEvent(self, event):
        if self._mouse_double_click_function is not None:
            self._mouse_double_click_function(event, self)

    def mousePressEvent(self, event):
        if self._mouse_press_function is not None:
            self._mouse_press_function(event, self)

    def mouseReleaseEvent(self, event):
        if self._mouse_release_function is not None:
            self._mouse_release_function(event, self)

    def hoverEnterEvent(self, event):
        if self._hover_enter_function is not None:
            self._hover_enter_function(event, self)

    def hoverLeaveEvent(self, event):
        if self._hover_leave_function is not None:
            self._hover_leave_function(event, self)

    def contextMenuEvent(self, event):
        if self._menu_function is not None:
            self._menu_function(event, self)


class Reaction(QGraphicsPolygonItem):
    def __init__(self, x, y,
                 flux=None, brush=None, pen=None,
                 invert_direction=False) -> None:

        self.x_coord = x
        self.y_coord = y
        self.ang = 0.0
        # Rot as side indicator
        self.db_representation: Any = None
        self.created: Union[datetime.datetime, None] = None
        self.assigned_es_id: db.ID
        self.spline: Optional[utils.bsplines.TrajectorySpline] = None
        self.barriers: Optional[Tuple[float, float]] = None  # lhs->ts, rhs->ts, in kJ/mol, direction as stored in db
        self.barrierless_type = False
        self.lhs_ids: List[str] = []
        self.rhs_ids: List[str] = []
        self.lhs_types: List[db.CompoundOrFlask] = []
        self.rhs_types: List[db.CompoundOrFlask] = []
        self.energy_difference: Union[None, float] = None
        # Keep track of the relative direction of the step and the reaction.
        self.invert_direction = invert_direction

        self.flux = flux
        self.allow_scaling = False
        poly = self.get_rotated_polygon()
        super().__init__(poly)
        self.__brush = brush
        self.__current_brush = brush
        self.__pen = pen
        self.__current_pen = pen
        self.reset_brush()
        if pen is not None:
            self.setPen(pen)
        self._mouse_release_function = None
        self._mouse_double_click_function = None
        self._mouse_press_function = None
        self._hover_enter_function = None
        self._hover_leave_function = None
        self._menu_function = None

    def get_scaling(self) -> float:
        from math import log10
        scaling = 1.0
        if self.flux is not None:
            flux = max(self.flux, 1e-9)
            scaling = max(1.0 / (1.0 + 0.5 * abs(log10(min(flux, 1.0)))), 0.1)
        return scaling

    def get_energy_difference(self) -> Union[None, float]:
        if not self.energy_difference:
            return None
        if self.invert_direction:
            return - self.energy_difference  # pylint: disable=invalid-unary-operand-type
        return self.energy_difference

    def reset_brush(self):
        if self.__brush is not None:
            self.__current_brush = self.__brush
            self.setBrush(self.__brush)
            self.__current_pen = self.__pen
            self.setPen(self.__pen)

    def set_current_brush(self, brush: QBrush):
        self.__current_brush = brush
        self.setBrush(brush)

    def get_current_brush(self):
        return self.__current_brush

    def get_brush(self) -> Union[None, QBrush]:
        return self.__brush

    def get_pen(self) -> Union[None, QPen]:
        return self.__pen

    def set_current_pen(self, pen: QPen):
        self.__current_pen = pen
        self.setPen(pen)

    def get_current_pen(self):
        return self.__current_pen

    def set_barriers(self, barriers: Union[Tuple[float, float], Tuple[None, None]]):
        if None in barriers:
            tmp_barriers = (0.0, 0.0)
        else:
            assert barriers[0] is not None and barriers[1] is not None
            tmp_barriers = barriers
        self.barriers = tmp_barriers

    def get_barriers(self):
        return self.barriers

    def set_created(self, created: datetime.datetime):
        self.created = created

    def get_created(self) -> Union[None, datetime.datetime]:
        return self.created

    def get_flux(self) -> Union[None, float]:
        return self.flux

    def __rot(self, x, y) -> QPoint:
        # Rotation relative to vector relative to (1, 0) - original position
        scaling = self.get_scaling() if self.allow_scaling else 1.0
        x = x * scaling
        y = y * scaling
        org_ang = np.arctan2(y, x)
        norm_xy = np.sqrt(x * x + y * y)
        dx = norm_xy * np.cos(self.ang + org_ang)
        dy = norm_xy * np.sin(self.ang + org_ang)
        # dy *= -1.0 if self.y_coord < 0 else 1.0
        return QPoint(int(self.x_coord + dx), int(self.y_coord - dy))

    def get_rotated_polygon(self) -> QPolygon:
        poly = QPolygon(
            [
                self.__rot(-15, +0),
                self.__rot(-5, +10),
                self.__rot(-5, +5),
                self.__rot(+5, +5),
                self.__rot(+5, +10),
                self.__rot(+15, +0),
                self.__rot(+5, -10),
                self.__rot(+5, -5),
                self.__rot(-5, -5),
                self.__rot(-5, -10),
                self.__rot(-15, +0),
            ]
        )

        return poly

    def outgoing(self) -> QPoint:
        return self.__rot(+15, +0)

    def incoming(self) -> QPoint:
        return self.__rot(-15, +0)

    def update_angle(self, angle) -> None:
        self.ang = angle
        poly = self.get_rotated_polygon()
        self.setPolygon(poly)

    def bind_mouse_release_function(self, func) -> None:
        self._mouse_release_function = func

    def bind_mouse_press_function(self, func) -> None:
        self._mouse_press_function = func

    def bind_mouse_double_click_function(self, func) -> None:
        self._mouse_double_click_function = func

    def bind_hover_enter_function(self, func) -> None:
        self._hover_enter_function = func

    def bind_hover_leave_function(self, func) -> None:
        self._hover_leave_function = func

    def mouseDoubleClickEvent(self, event) -> None:
        if self._mouse_double_click_function is not None:
            self._mouse_double_click_function(event, self)

    def bind_menu_function(self, func):
        self._menu_function = func

    def mousePressEvent(self, event) -> None:
        if self._mouse_press_function is not None:
            self._mouse_press_function(event, self)

    def mouseReleaseEvent(self, event) -> None:
        if self._mouse_release_function is not None:
            self._mouse_release_function(event, self)

    def hoverEnterEvent(self, event) -> None:
        if self._hover_enter_function is not None:
            self._hover_enter_function(event, self)

    def hoverLeaveEvent(self, event) -> None:
        if self._hover_leave_function is not None:
            self._hover_leave_function(event, self)

    def contextMenuEvent(self, event):
        if self._menu_function is not None:
            self._menu_function(event, self)


class Pathinfo(QGraphicsTextItem):
    def __init__(self, x, y, path: List[Any], path_rank: int, path_length: float, text="") -> None:
        super().__init__(text)
        self.x_coord = x
        self.y_coord = y
        self.center()
        self.text = text
        self.font_size = 30
        self.font_family = 'Arial'
        self.setFont(QFont(self.font_family, self.font_size))
        self.setDefaultTextColor(qcolor_by_key("primaryTextColor"))
        font_metrics = QFontMetrics(QFont(self.font_family, self.font_size))
        # Y position such that two line text is centered at line break
        self.setPos(x, y - 1 * self.font_size - int(0.5 * font_metrics.lineSpacing()))

        self.path = path
        self.path_rank = path_rank
        self.path_length = path_length
        self.setToolTip("Double click for energy diagram")
        # Function attributes
        self._mouse_release_function = None
        self._mouse_double_click_function = None
        self._mouse_press_function = None
        self._hover_enter_function = None
        self._hover_leave_function = None
        self._menu_function = None

    def center(self):
        return QPoint(self.x_coord, self.y_coord)

    def bind_mouse_release_function(self, func) -> None:
        self._mouse_release_function = func

    def bind_mouse_press_function(self, func) -> None:
        self._mouse_press_function = func

    def bind_mouse_double_click_function(self, func) -> None:
        self._mouse_double_click_function = func

    def bind_hover_enter_function(self, func) -> None:
        self._hover_enter_function = func

    def bind_hover_leave_function(self, func) -> None:
        self._hover_leave_function = func

    def mouseDoubleClickEvent(self, event) -> None:
        if self._mouse_double_click_function is not None:
            self._mouse_double_click_function(event, self)

    def bind_menu_function(self, func):
        self._menu_function = func

    def mousePressEvent(self, event) -> None:
        if self._mouse_press_function is not None:
            self._mouse_press_function(event, self)

    def mouseReleaseEvent(self, event) -> None:
        if self._mouse_release_function is not None:
            self._mouse_release_function(event, self)

    def hoverEnterEvent(self, event) -> None:
        if self._hover_enter_function is not None:
            self._hover_enter_function(event, self)

    def hoverLeaveEvent(self, event) -> None:
        if self._hover_leave_function is not None:
            self._hover_leave_function(event, self)

    def contextMenuEvent(self, event):
        if self._menu_function is not None:
            self._menu_function(event, self)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

import typing

if typing.TYPE_CHECKING:
    Signal = typing.Any
else:
    pass


from PySide2.QtCore import Qt
from PySide2.QtWidgets import (
    QDockWidget,
    QHBoxLayout,
    QLabel,
    QLineEdit,
    QPushButton,
    QSpinBox,
    QVBoxLayout,
    QWidget,
    QDoubleSpinBox,
    QCheckBox)

from scine_heron.containers.combo_box import BaseBox
from scine_heron.autocas.autocas_settings import AutocasSettings
# from scine_heron.autocas.basic_options import Interfaces
# from scine_heron.autocas.open_molcas_popup import OpenMolcasOptionsPopup
# from scine_heron.autocas.options_status_manager import OptionsStatusManager
from scine_heron.autocas.signal_handler import SignalHandler
# from scine_heron.settings.settings import LabelsStyle, MoleculeStyle
# from scine_heron.status_manager import Status, StatusManager

import os


class SpinBox(QSpinBox):
    def wheelEvent(self, event):
        event.ignore()


class DoubleSpinBox(QDoubleSpinBox):
    def wheelEvent(self, event):
        event.ignore()


class ComboBox(BaseBox):
    def wheelEvent(self, event):
        event.ignore()


class OptionsWidget(QDockWidget):
    # pylint: disable-next=W0613
    def __init__(
        self,
        parent,
        # options_status_manager: OptionsStatusManager,
        signal_handler: SignalHandler,
        autocas_settings: AutocasSettings,
    ):
        QDockWidget.__init__(self, "")

        self.signal_handler = signal_handler
        self.autocas_settings = autocas_settings

        self.setFeatures(QDockWidget.NoDockWidgetFeatures)
        self.__dockedWidget = QWidget(self)
        self.setWidget(self.__dockedWidget)
        self.__layout = QVBoxLayout()
        self.__dockedWidget.setLayout(self.__layout)
        self.__layout.setAlignment(Qt.AlignTop)

        self.button_apply_settings = QPushButton("Apply Settings")
        self.__layout.addWidget(self.button_apply_settings)
        # pylint: disable-next=E1101
        self.button_apply_settings.clicked.connect(
            self.get_options
        )

        self.molecular_options()
        self.autocas_options()
        self.molcas_options()
        self.initial_orbital_options()
        self.initial_dmrg_options()
        self.final_calculation_options()

    def get_options(self):
        # molecular options
        self.autocas_settings.interface_basis_set = self.basis_set_edit.text()
        self.autocas_settings.molecule_charge = int(self.charge_edit.text())
        self.autocas_settings.molecule_spin_multiplicity = int(self.spin_multiplicity_edit.text())
        self.autocas_settings.interface_n_excited_states = int(self.number_of_roots_edit.text())
        self.autocas_settings.molecule_double_d_shell = bool(self.double_d_shell.text())

        # initial orbital options
        self.autocas_settings.interface_uhf = bool(self.uhf.text())

        # molcas options
        self.autocas_settings.molcas_project_name = self.molcas_project_name.text()
        self.autocas_settings.molcas_dump = bool(self.molcas_dump.text())
        self.autocas_settings.molcas_work_dir = self.molcas_work_dir.text()
        self.autocas_settings.interface_cholesky = bool(self.cholesky.text())

        # # initial dmrg options
        self.autocas_settings.interface_method = self.method.text()
        self.autocas_settings.interface_dmrg_bond_dimension = int(self.bond_dimension.text())
        self.autocas_settings.interface_dmrg_sweeps = int(self.n_sweeps.text())
        self.autocas_settings.interface_fiedler = bool(self.fiedler_ordering.text())
        self.autocas_settings.enable_large_cas = bool(self.large_cas.text())
        self.autocas_settings.large_spaces_max_orbitals = int(self.n_large_cas_orbitals.text())
        #
        # # final calculation options
        # self.final_method
        # self.final_bond_dimension
        # self.final_n_sweeps
        # self.final_fiedler_ordering
        # self.final_post_method
        # self.final_ipea
        #
        # # autocas options
        # self.plateau_values
        # self.threshold_step
        # self.weak_correlation_threshold
        # self.single_reference_threshold

    def molecular_options(self):
        self.basis_set_edit = QLineEdit(self)
        self.basis_set_edit.setText("cc-pvdz")

        self.charge_edit = SpinBox()
        self.charge_edit.setValue(0)

        self.spin_multiplicity_edit = SpinBox()
        self.spin_multiplicity_edit.setValue(1)
        self.spin_multiplicity_edit.setMinimum(1)

        self.number_of_roots_edit = SpinBox()
        self.number_of_roots_edit.setValue(1)
        self.number_of_roots_edit.setMinimum(1)

        self.double_d_shell = QCheckBox()
        self.double_d_shell.setChecked(True)

        self.__layout.addWidget(QLabel("Molecular Settings"))
        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Basis Set"))
        sub_layout.addWidget(QLabel("Charge"))
        sub_layout.addWidget(QLabel("Spin Multiplicity"))
        sub_layout.addWidget(QLabel("Number Of Roots"))
        sub_layout.addWidget(QLabel("Include double d-shell"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.basis_set_edit)
        sub_layout.addWidget(self.charge_edit)
        sub_layout.addWidget(self.spin_multiplicity_edit)
        sub_layout.addWidget(self.number_of_roots_edit)
        sub_layout.addWidget(self.double_d_shell)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def initial_orbital_options(self):
        self.__layout.addWidget(QLabel("Initial Orbital Settings"))

        self.uhf = QCheckBox()
        self.uhf.setChecked(False)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Unrestricted"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.uhf)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def molcas_options(self):
        self.__layout.addWidget(QLabel("Molcas Settings"))

        self.molcas_project_name = QLineEdit(self)
        self.molcas_project_name.setText("autocas")

        self.molcas_dump = QCheckBox()
        self.molcas_dump.setChecked(True)

        self.molcas_work_dir = QLineEdit(self)
        self.molcas_work_dir.setText(os.getcwd() + "/test")

        self.cholesky = QCheckBox()
        self.cholesky.setChecked(True)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Project Name"))
        sub_layout.addWidget(QLabel("Enable Dump"))
        sub_layout.addWidget(QLabel("Work Dir"))
        sub_layout.addWidget(QLabel("Enable Cholesky decomposition"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.molcas_project_name)
        sub_layout.addWidget(self.molcas_dump)
        sub_layout.addWidget(self.molcas_work_dir)
        sub_layout.addWidget(self.cholesky)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def initial_dmrg_options(self):
        self.__layout.addWidget(QLabel("Initial DMRG Settings"))

        self.method = ComboBox()
        self.method.addItems(["DMRG-CI", "DMRG-SCF"])

        self.bond_dimension = SpinBox()
        self.bond_dimension.setMaximum(6000)
        self.bond_dimension.setMinimum(1)
        self.bond_dimension.setValue(250)

        self.n_sweeps = SpinBox()
        # self.n_sweeps.scroll
        self.n_sweeps.setValue(5)
        self.n_sweeps.setMinimum(1)

        self.fiedler_ordering = QCheckBox()
        self.fiedler_ordering.setChecked(True)

        self.large_cas = QCheckBox()
        self.large_cas.setChecked(False)

        self.n_large_cas_orbitals = SpinBox()
        self.n_large_cas_orbitals.setMinimum(1)
        self.n_large_cas_orbitals.setValue(10)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Method"))
        sub_layout.addWidget(QLabel("Bond dimension"))
        sub_layout.addWidget(QLabel("Number of Sweeps"))
        sub_layout.addWidget(QLabel("Fiedler Ordering"))
        sub_layout.addWidget(QLabel("Enable Large CAS Protocol"))
        sub_layout.addWidget(QLabel("Number of orbitals per sub-CAS"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.method)
        sub_layout.addWidget(self.bond_dimension)
        sub_layout.addWidget(self.n_sweeps)
        sub_layout.addWidget(self.fiedler_ordering)
        sub_layout.addWidget(self.large_cas)
        sub_layout.addWidget(self.n_large_cas_orbitals)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def final_calculation_options(self):
        self.__layout.addWidget(QLabel("Final Calculation Settings"))

        self.final_method = ComboBox()
        self.final_method.addItems(["DMRG-CI", "DMRG-SCF", "CASSCF", "CASCI"])

        self.final_bond_dimension = SpinBox()
        self.final_bond_dimension.setMaximum(6000)
        self.final_bond_dimension.setMinimum(1)
        self.final_bond_dimension.setValue(2000)

        self.final_n_sweeps = SpinBox()
        self.final_n_sweeps.setValue(20)
        self.final_n_sweeps.setMinimum(1)

        self.final_fiedler_ordering = QCheckBox()
        self.final_fiedler_ordering.setChecked(True)

        self.final_post_method = ComboBox()
        self.final_post_method.addItems(["CASPT2", "NEVPT2"])

        self.final_ipea = DoubleSpinBox()
        self.final_ipea.setValue(0.0)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Method"))
        sub_layout.addWidget(QLabel("Bond dimension"))
        sub_layout.addWidget(QLabel("Number of Sweeps"))
        sub_layout.addWidget(QLabel("Fiedler Ordering"))
        sub_layout.addWidget(QLabel("Post CAS method"))
        sub_layout.addWidget(QLabel("IPEA-Shift"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.final_method)
        sub_layout.addWidget(self.final_bond_dimension)
        sub_layout.addWidget(self.final_n_sweeps)
        sub_layout.addWidget(self.final_fiedler_ordering)
        sub_layout.addWidget(self.final_post_method)
        sub_layout.addWidget(self.final_ipea)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def autocas_options(self):
        self.__layout.addWidget(QLabel("AutoCAS Settings"))

        self.plateau_values = SpinBox(self)
        self.plateau_values.setValue(10)

        self.threshold_step = DoubleSpinBox()
        self.threshold_step.setValue(0.01)

        self.weak_correlation_threshold = DoubleSpinBox()
        self.weak_correlation_threshold.setValue(0.02)

        self.single_reference_threshold = DoubleSpinBox()
        self.single_reference_threshold.setValue(0.14)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Required values for a plateau"))
        sub_layout.addWidget(QLabel("Step size for a plateau"))
        sub_layout.addWidget(QLabel("Weak correlation threshold"))
        sub_layout.addWidget(QLabel("Single reference threshold"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(self.plateau_values)
        sub_layout.addWidget(self.threshold_step)
        sub_layout.addWidget(self.weak_correlation_threshold)
        sub_layout.addWidget(self.single_reference_threshold)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

    def plot_options(self):
        self.__layout.addWidget(QLabel("Initial Orbital Settings"))

        basis_set_edit = QLineEdit(self)
        basis_set_edit.setText("cc-pvdz")

        charge_edit = SpinBox()
        charge_edit.setValue(0)
        charge_edit.setMinimum(-100000)

        spin_multiplicity_edit = SpinBox()
        spin_multiplicity_edit.setValue(1)
        spin_multiplicity_edit.setMinimum(1)

        number_of_roots_edit = SpinBox()
        number_of_roots_edit.setValue(1)
        number_of_roots_edit.setMinimum(1)

        layout = QHBoxLayout()
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(QLabel("Basis Set"))
        sub_layout.addWidget(QLabel("Charge"))
        sub_layout.addWidget(QLabel("Spin Multiplicity"))
        sub_layout.addWidget(QLabel("Number Of Roots"))
        layout.addLayout(sub_layout)
        sub_layout = QVBoxLayout()
        sub_layout.addWidget(basis_set_edit)
        sub_layout.addWidget(charge_edit)
        sub_layout.addWidget(spin_multiplicity_edit)
        sub_layout.addWidget(number_of_roots_edit)
        layout.addLayout(sub_layout)
        self.__layout.addLayout(layout)

        # self.setFeatures(QDockWidget.NoDockWidgetFeatures)
        #
        # self.__dockedWidget = QWidget(self)
        # self.setWidget(self.__dockedWidget)
        #
        # self.__layout = QVBoxLayout()
        # self.__dockedWidget.setLayout(self.__layout)
        # self.__layout.setAlignment(Qt.AlignTop)
        #
        # # self.__options_status_manager = options_status_manager
        #
        # self.__widgets_dict: Dict[str, Any] = {}
        # self.__widget_height = 30
        # self.__widget_width = 130
        #
        # self.__enabled = StatusManager(True)
        #
        # self.__add_interface_option(
        #     "Interface",
        #     [Interfaces.OpenMolcas.value],  # , Interfaces.Serenity.value],
        #     self.__update_interface,
        #     Interfaces.OpenMolcas.value,
        #     enabled=self.__enabled,
        # )
        # self.__add_system_options_at_layout(enabled=self.__enabled)

    # def button_clicked(self, s):
    #     open_molcas_popup = OpenMolcasOptionsPopup(self)
    #     open_molcas_popup.exec_()
    #
    # def __add_interface_option(
    #     self,
    #     option_name: str,
    #     all_values: List[str],
    #     update: Callable[[List[str], int], None],
    #     default_value: str,
    #     enabled: Status[bool],
    # ) -> None:
    #     combo_box = self.__add_combo_box_at_layout(option_name, option_name, all_values)
    #     combo_box.currentIndexChanged.connect(partial(update, all_values))
    #     combo_box.setCurrentIndex(all_values.index(default_value))
    #     enabled.changed_signal.connect(combo_box.setEnabled)
    #
    #     self.interface_options = QPushButton("Advanced Options")
    #     self.__layout.addWidget(self.interface_options)
    #     self.interface_options.clicked.connect(self.button_clicked)
    #
    # def __add_system_options_at_layout(self, enabled: Status[bool]):
    #     basis_set_edit = QLineEdit(self)
    #     basis_set_edit.setText("cc-pvdz")
    #
    #     charge_edit = QSpinBox()
    #     charge_edit.setValue(0)
    #     charge_edit.setMinimum(-100000)
    #
    #     spin_multiplicity_edit = QSpinBox()
    #     spin_multiplicity_edit.setValue(1)
    #     spin_multiplicity_edit.setMinimum(1)
    #
    #     number_of_roots_edit = QSpinBox()
    #     number_of_roots_edit.setValue(1)
    #     number_of_roots_edit.setMinimum(1)
    #
    #     layout = QHBoxLayout()
    #     sub_layout = QVBoxLayout()
    #     sub_layout.addWidget(QLabel("Basis Set"))
    #     sub_layout.addWidget(QLabel("Charge"))
    #     sub_layout.addWidget(QLabel("Spin Multiplicity"))
    #     sub_layout.addWidget(QLabel("Number Of Roots"))
    #     layout.addLayout(sub_layout)
    #     sub_layout = QVBoxLayout()
    #     sub_layout.addWidget(basis_set_edit)
    #     sub_layout.addWidget(charge_edit)
    #     sub_layout.addWidget(spin_multiplicity_edit)
    #     sub_layout.addWidget(number_of_roots_edit)
    #     layout.addLayout(sub_layout)
    #     self.__layout.addLayout(layout)
    #
    # def __update_interface(self, all_values: List[str], index: int) -> None:
    #     """
    #     Update molecule style.
    #     """
    #     self.__options_status_manager.interface = Interfaces(all_values[index])
    #
    # def __add_combo_box_at_layout(
    #     self,
    #     setting_name: str,
    #     setting_key: str,
    #     all_values: List[str],
    # ) -> QComboBox:
    #     """
    #     Add QComboBox widget.
    #     setting_name is a setting display name.
    #     setting_key is a setting name in sparrow.
    #     all_values is a list of valid values.
    #     """
    #     combo_box = QComboBox()
    #     combo_box.addItems(all_values)
    #     combo_box.setFixedSize(QSize(self.__widget_width, self.__widget_height + 1))
    #
    #     #   self.__widgets_dict[setting_key] = combo_box
    #
    #     layout = QHBoxLayout()
    #     layout.addWidget(QLabel(setting_name))
    #     layout.addWidget(combo_box)
    #     self.__layout.addLayout(layout)
    #
    #     return combo_box
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #
    #

# class SectionExpandButton(QPushButton):
#     def __init__(self, item, text="", parent=None):
#         super().__init__(text, parent)
#         self.section = item
#         self.clicked.connect(self.on_clicked)
#
#     def on_clicked(self):
#         if self.section.isExpanded():
#             self.section.setExpanded(False)
#         else:
#             self.section.setExpanded(True)
#
#
# class OptionsWidget_bck(QDialog):
#     def __init__(
#         self,
#         parent: QWidget,
#         signal_handler: SignalHandler,
#         autocas_settings: AutocasSettings,
#     ):
#         super().__init__()
#         self.signal_handler = signal_handler
#         self.autocas_settings = autocas_settings
#         self.tree = QTreeWidget()
#         self.tree.setHeaderHidden(True)
#         layout = QVBoxLayout()
#         layout.addWidget(self.tree)
#         self.setLayout(layout)
#         # self.tree.setIndentation(0)
#
#         self.sections: List[Tuple[QWidget, str]] = []
#         self.define_sections()
#         self.add_sections()
#
#     def add_sections(self):
#         """adds a collapsible sections for every
#         (title, widget) tuple in self.sections
#         """
#         for (title, widget) in self.sections:
#             button1 = self.add_button(title)
#             section1 = self.add_widget(button1, widget)
#             button1.addChild(section1)
#
#     def define_sections(self):
#         """reimplement this to define all your sections
#         and add them as (title, widget) tuples to self.sections
#         """
#         widget = QFrame(self.tree)
#         layout = QHBoxLayout(widget)
#         layout.addWidget(
#             OrbitalOption(self, self.signal_handler, self.autocas_settings)
#         )
#         # layout.addWidget(QLabel("Bla"))
#         # layout.addWidget(QLabel("Blubb"))
#         self.sections.append(("Hartree-Fock", widget))
#         self.sections.append(("DMRG", widget))
#
#     def add_button(self, title):
#         """creates a QTreeWidgetItem containing a button
#         to expand or collapse its section
#         """
#         item = QTreeWidgetItem()
#         self.tree.addTopLevelItem(item)
#         self.tree.setItemWidget(item, 0, SectionExpandButton(item, text=title))
#         return item
#
#     def add_widget(self, button, widget):
#         """creates a QWidgetItem containing the widget,
#         as child of the button-QWidgetItem
#         """
#         section = QTreeWidgetItem(button)
#         section.setDisabled(True)
#         self.tree.setItemWidget(section, 0, widget)
#         return section
#
#
# class OrbitalOption(QDockWidget):
#     def __init__(self, parent, signal_handler, autocas_settings):
#         super().__init__()
#         self.setFeatures(QDockWidget.NoDockWidgetFeatures)
#
#         self.__dockedWidget = QWidget(self)
#         self.setWidget(self.__dockedWidget)
#
#         self.__layout = QVBoxLayout()
#         self.__dockedWidget.setLayout(self.__layout)
#         self.__layout.setAlignment(Qt.AlignTop)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import Optional, List

from PySide2.QtWidgets import QGridLayout, QWidget, QTabWidget

from scine_heron.autocas.autocas_controller import AutocasController
from scine_heron.autocas.autocas_settings import AutocasSettings
from scine_heron.autocas.autocas_wrapper import AutocasWrapper
from scine_heron.autocas.file_tree_widget import FileTreeWidget
from scine_heron.autocas.options import OptionsWidget
from scine_heron.autocas.output_widget import OutputWidget
from scine_heron.autocas.signal_handler import SignalHandler
#  from scine_heron.autocas.toolbar_widget import ToolbarWidget
from scine_heron.autocas.autocas_project_widget import AutocasProjectWidget
from scine_heron.electronic_data.orbital_groups import OrbitalGroupMap
from scine_heron.electronic_data.orbital_group_file_reader import OrbitalGroupFileReader


class AutocasWidget(QWidget):
    def __init__(self, parent: QWidget) -> None:
        QWidget.__init__(self, parent)
        self.__all_projects: List[AutocasProjectWidget] = []
        self.project_tabs = QTabWidget()
        self.__add_project_tab()
        self.autocas_settings = self.__singular_project().autocas_settings
        self.signal_handler = self.__singular_project().signal_handler
        # toolbar
        # self.tool_bar = ToolbarWidget(self, self.signal_handler, self.autocas_settings)
        # expandable file tree
        self.file_tree = FileTreeWidget(
            self, self.signal_handler, self.autocas_settings
        )

        # shows stdout
        self.output = OutputWidget(self)
        # handles all autocas options
        self.options = OptionsWidget(self, self.signal_handler, self.autocas_settings)
        # handles autocas runs
        self.autocas_controler = AutocasController(self, self.signal_handler)
        # autocas instance
        self.autocas = AutocasWrapper(self.autocas_settings, self.signal_handler)

        # set layout
        self.__layout = QGridLayout()
        # from  height, width -> to height, width
        # self.__layout.addWidget(self.tool_bar, 0, 0)
        self.__layout.addWidget(self.file_tree, 1, 0, 2, 1)
        self.__layout.addWidget(self.project_tabs, 1, 1, 1, 3)
        # self.__layout.addWidget(self.result, 1, 1)  # , 2, 2)
        self.__layout.addWidget(self.output, 2, 1)  # , 3, 3)
        # self.__layout.addWidget(self.mo_diagram, 1, 3)  # , 2, 4)
        self.__layout.addWidget(self.autocas_controler, 2, 3)
        # self.file_tree.setMaximumWidth(int(self.tool_bar.width() * 0.55))
        self.setLayout(self.__layout)

        self.signal_handler.load_orbital_groups_file_signal.connect(self.__load_orbital_groups_file)
        self.__orbital_map: Optional[OrbitalGroupMap] = None

    def __singular_project(self):
        assert self.__all_projects
        return self.__all_projects[0]

    def __add_project_tab(self):
        n_projects = len(self.__all_projects)
        signal_handler = SignalHandler(self)
        autocas_settings = AutocasSettings()
        label = "Project " + str(n_projects)
        project_tab = AutocasProjectWidget(self, signal_handler, autocas_settings, label)
        self.project_tabs.addTab(project_tab, label)
        self.__all_projects.append(project_tab)

    def __load_all_xyz_files(self):
        for project in self.__all_projects:
            # print(project.autocas_settings.molecule_xyz_file)
            project.signal_handler.load_xyz_file_signal.emit(project.autocas_settings.molecule_xyz_file)

    def __load_all_orbital_files(self):
        for project in self.__all_projects:
            project.signal_handler.load_molden_file_signal.emit(project.autocas_settings.molcas_orbital_file)

    def __add_orbital_map_to_settings(self):
        for i, project in enumerate(self.__all_projects):
            project.autocas_settings.orbital_index_sets = self.__orbital_map.get_index_sets_for_system(i)

    def __load_orbital_groups_file(self, file_name: str):
        self.__orbital_map = OrbitalGroupFileReader.read_orbital_group_file(file_name)
        assert self.__orbital_map
        for _ in range(self.__orbital_map.get_n_systems() - 1):
            self.__add_project_tab()
        self.__add_orbital_map_to_settings()
        self.__load_all_xyz_files()
        self.__load_all_orbital_files()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import Optional, Any, Callable, TYPE_CHECKING
from pkgutil import iter_modules
import os

from PySide2.QtGui import QKeySequence, QIcon
from PySide2.QtWidgets import (
    QAction,
    QToolBar,
    QErrorMessage,
    QWidget,
    QLabel,
    QStyle,
)
from PySide2.QtCore import QObject, QSize
if TYPE_CHECKING:
    Signal = Any
else:
    from PySide2.QtCore import Signal

import scine_heron.config as config
from scine_heron.resources import resource_path
from scine_heron.toolbar.about_dialog import AboutDialog


class HeronToolBar(QToolBar):
    """
    Simplifying some interaction with the QToolBar
    """

    def __init__(self, parent: Optional[QObject] = None):
        super(HeronToolBar, self).__init__(parent=parent)

    def shortened_add_action(self, icon_cmd, text: str, shortcut: str, function: Callable) -> QAction:
        if shortcut:
            q_shortcut = QKeySequence(shortcut)
            description_string = f'{text} ({q_shortcut.toString()})</p>'
        else:
            description_string = f'{text}</p>'
        if isinstance(icon_cmd, str):
            icon = QIcon(os.path.join(resource_path(), 'icons', icon_cmd))
        else:
            icon = self.style().standardIcon(icon_cmd)

        action = self.addAction(
            icon,
            self.tr(
                f'<p>{description_string}'  # type: ignore[arg-type]
            ),
        )
        if shortcut:
            action.setShortcut(q_shortcut)
        action.triggered.connect(function)  # pylint: disable=no-member
        # the following lines are necessary due to a bug in qt-widgets
        # we have to overwrite the size and stylesheet
        # because the height is incorrectly defined for QToolButtons
        # that are part of a toolbar that is set in some widgets / layouts
        # such as the DictOptionsWidget
        tool_button = self.widgetForAction(action)
        size = tool_button.size()
        # double widths is not a typo, but chosen
        # because the height is ill-defined
        tool_button.setFixedSize(QSize(int(round(size.width() * 0.75)), int(round(size.width() * 0.75))))
        style = f"""
           QToolButton {{ background-color: none;}}
           QToolButton:hover {{ background-color: {config.COLORS['secondaryDarkColor']};}}
        """
        tool_button.setStyleSheet(style)
        self.updateGeometry()

        return action


class ToolBarWithSaveLoad(HeronToolBar):

    def __init__(self, save_call: Callable, load_call: Callable, parent: Optional[QObject] = None,
                 hover_text_save="Save Setup", hover_text_load="Save Setup"):
        super().__init__(parent=parent)

        self.shortened_add_action(
            QStyle.SP_DialogSaveButton, hover_text_save, "Ctrl+S", save_call
        )
        self.shortened_add_action(
            QStyle.SP_DialogOpenButton, hover_text_load, "Ctrl+O", load_call
        )


class IOToolbar(HeronToolBar):
    """
    A toolbar determining the possible main tabs
    """
    connect_db_signal = Signal(QWidget, QWidget, QWidget, QWidget, QWidget, QWidget)

    def __init__(self, parent: Optional[QObject] = None):
        super(IOToolbar, self).__init__(parent=parent)
        self.db_manager: Optional[Any] = None

        self.shortened_add_action("db.png", "Connect to Database", "Ctrl+D",
                                  self.db_connection)
        self.__db_label = QLabel("Database: Unavailable")
        if "scine_database" in (name for _, name, _ in iter_modules()):
            self.__db_label.setText("Database: Disconnected")
        self.addWidget(self.__db_label)
        self.shortened_add_action("db.png", "Reaction Templates", "Ctrl+T",
                                  self.__art_view)
        self.__art_label = QLabel("Reaction Templates: Unavailable")
        self.template_storage = None
        if "scine_art" in (name for _, name, _ in iter_modules()):
            from scine_heron.reaction_templates.reaction_template_storage import ReactionTemplateStorage
            self.template_storage = ReactionTemplateStorage()
            self.__art_label.setText("Reaction Templates: 0")
            self.template_storage.reaction_template_count_changed.connect(self.__update_template_count)
            self.__rtd = None
        self.addWidget(self.__art_label)
        self.__about_label = QLabel("About")
        self.shortened_add_action(QStyle.SP_TitleBarContextHelpButton, "About Heron", "Ctrl+H",
                                  self.__about_view)
        self.addWidget(self.__about_label)
        self.__about = None

    def generate_db_manager(self, db_name: str, ip: str, port: int) -> None:
        if "scine_database" in (name for _, name, _ in iter_modules()):
            import scine_database
            self.db_manager = scine_database.Manager()
            credentials = scine_database.Credentials(ip, port, db_name)
            self.db_manager.set_credentials(credentials)
        else:
            self.db_manager = None
            return

    def current_credentials(self) -> Optional[Any]:
        if self.db_manager is None:
            return None
        return self.db_manager.get_credentials()

    def __art_view(self):
        if "scine_art" in (name for _, name, _ in iter_modules()):
            from scine_heron.reaction_templates.reaction_template_dialog import ReactionTemplateDialog
            self.__rtd = ReactionTemplateDialog(self.parent().parent(), self.template_storage)
            self.__rtd.show()

    def __about_view(self):
        self.__about = AboutDialog(self.parent().parent())
        self.__about.show()

    def __update_template_count(self, count: int) -> None:
        self.__art_label.setText(f"Reaction Templates: {count}")

    def db_connection(self, silently: bool = False) -> None:
        """
        Opens dialog to connect to a SCINE DB
        """
        if "scine_database" in (name for _, name, _ in iter_modules()):
            from scine_heron.database.monitor_widget import DatabaseMonitorWidget
            from scine_heron.database.reaction_compound_widget import CompoundReactionWidget
            from scine_heron.database.database_viewer_widget import DatabaseViewerWidget
        else:
            error_dialog = QErrorMessage(parent=self)
            error_dialog.setWindowTitle("Missing SCINE Database Wrapper")
            error_dialog.showMessage(
                "The SCINE Database Wrapper could not be detected. Connections \
                    to a SCINE Database require it to be installed.\n"
            )
            return

        from scine_heron.database.connection_dialog import DatabaseConnectionDialog

        if self.db_manager is not None:
            old_credentials = self.db_manager.get_credentials()
            old_connection = self.db_manager.is_connected()
        else:
            old_credentials = None
            old_connection = False
        if silently:
            try:
                self.db_manager.connect(expect_initialized_db=True)
            except RuntimeError:
                # cannot connect directly, call the dialog, which has better handling
                silently = False
        if not silently:
            connector = DatabaseConnectionDialog(parent=self, db_manager=self.db_manager)
            self.db_manager = connector.get_db_manager()
        new_credentials = self.db_manager.get_credentials() != old_credentials
        newly_connected = self.db_manager.is_connected() and not old_connection

        chemoton_widget = None
        steering_widget = None
        if (self.db_manager.is_connected() and new_credentials) or newly_connected:
            if "scine_chemoton" in (name for _, name, _ in iter_modules()):
                from scine_heron.chemoton.chemoton_widget_container import ChemotonWidgetContainer
                from scine_heron.chemoton.create_chemoton_widget import CreateEngineWidget
                from scine_chemoton import gears
                chemoton_widget = ChemotonWidgetContainer(self, self.db_manager, [gears.Gear], [CreateEngineWidget])
            if "scine_chemoton" in (name for _, name, _ in iter_modules()):
                from scine_heron.steering.display_widget import SteeringDisplay
                from scine_heron.chemoton.create_chemoton_widget import CreateStep, CreateSelection
                from scine_chemoton.steering_wheel.network_expansions import NetworkExpansion
                from scine_chemoton.steering_wheel.selections import Selection
                steering_widget = SteeringDisplay(self, self.db_manager, [Selection, NetworkExpansion],
                                                  [CreateSelection, CreateStep])
            self.__db_label.setText("Database: Connecting")
            kinetic_exploration_widget = None
            # Kinetix will be disabled until feature complete
            # calculations = self.db_manager.get_collection("calculations")
            # if calculations.get_one_calculation(dumps({"job.order": "kinetx_kinetic_modeling"})):
            #     kinetic_exploration_widget = KineticExplorationProgressWidget(self, self.db_manager)
            self.connect_db_signal.emit(
                CompoundReactionWidget(self, self.db_manager),
                DatabaseMonitorWidget(self, self.db_manager),
                DatabaseViewerWidget(self, self.db_manager),
                chemoton_widget,
                steering_widget,
                kinetic_exploration_widget
            )
            self.__db_label.setText("Database: Connected")
        elif self.db_manager.is_connected():
            self.__db_label.setText("Database: Connected")
        else:
            self.connect_db_signal.emit(None, None, None, None, None, None)
            self.__db_label.setText("Database: Disconnected")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import List, Union

# Third party imports
import scine_database as db

from warnings import warn
warn('This is deprecated, please import the queries from scine_database', DeprecationWarning, stacklevel=2)


def query_concentration(label: str, centroid: db.Structure, properties: db.Collection) -> float:
    """
    Query a concentration for a given structure.

    Parameters
    ----------
    label : str
        The concentration property label.
    centroid : db.Structure
        The structure as a linked object.
    properties : db.Collection
        The property collection.

    Returns
    -------
    float
        The concentration according to property label. Return 0.0 if no property is present.
    """
    property_list = centroid.get_properties(label)
    if not property_list:
        return 0.0
    # pick last property if multiple
    prop = db.NumberProperty(property_list[-1], properties)
    return prop.get_data()


def query_concentration_list(label: str, centroid: db.Structure, properties: db.Collection) -> List[float]:
    """
    Query all concentrations of a given type for a structure.

    Parameters
    ----------
    label : str
        The concentration property label.
    centroid : db.Structure
        The structure as a linked object.
    properties : db.Collection
        The property collection.

    Returns
    -------
    List[float]
        The concentrations according to property label. Returns an empty list if no property is present.
    """
    property_list = centroid.get_properties(label)
    concentrations = list()
    for generic_property in property_list:
        prop = db.NumberProperty(generic_property, properties)
        concentrations.append(prop.get_data())
    return concentrations


def query_reaction_flux(label_post_fix: str, reaction: db.Reaction, compounds: db.Collection, flasks: db.Collection,
                        structures: db.Collection, properties: db.Collection):
    """
    Query the concentration flux for a given reaction.

    Parameters
    ----------
    label_post_fix : str
        The property label for the concentration flux will be given as reaction.id().string() + label_post_fix
    reaction : db.Reaction
        The reaction.
    compounds : db.Collection
        The compound collection.
    flasks : db.Collection
        The flask collection.
    structures : db.Collection
        The structure collection.
    properties : db.Collection
        The property collection.

    Returns
    -------
    float
        The concentration flux along the reaction. If no flux is given in the database, 0.0 is returned.
    """
    label = reaction.id().string() + label_post_fix
    a_id = reaction.get_reactants(db.Side.LHS)[0][0]
    a_type = reaction.get_reactant_types(db.Side.LHS)[0][0]
    if a_type == db.CompoundOrFlask.FLASK:
        aggregate: Union[db.Compound, db.Flask] = db.Flask(a_id, flasks)
    else:
        assert a_type == db.CompoundOrFlask.COMPOUND
        aggregate = db.Compound(a_id, compounds)
    structure = db.Structure(aggregate.get_centroid(), structures)
    return query_concentration(label, structure, properties)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import Optional

from scine_heron.autocas.mo_diagram import MODiagram
from scine_heron.autocas.result_widget import ResultWidget
from scine_heron.autocas.signal_handler import SignalHandler
from scine_heron.autocas.autocas_settings import AutocasSettings
import scine_heron.utilities as utils

from PySide2.QtWidgets import QHBoxLayout, QWidget, QMenu


class AutocasProjectWidget(QWidget):
    def __init__(self, parent: Optional[QWidget], signal_handler: SignalHandler, autocas_settings: AutocasSettings,
                 label: Optional[str] = None, single_window: bool = False):
        QWidget.__init__(self, parent)

        self.signal_handler = signal_handler
        self.autocas_settings = autocas_settings

        self.__layout = QHBoxLayout()
        self.__results_widget = ResultWidget(self, self.signal_handler, self.autocas_settings)
        self.__mo_diagram_widget = utils.vertical_scroll_area_wrap(
            MODiagram(self, self.signal_handler, self.autocas_settings))

        self.__layout.addWidget(self.__results_widget)
        self.__layout.addWidget(self.__mo_diagram_widget)
        self.setLayout(self.__layout)
        self.__expanded_view: Optional[QWidget] = None
        self.__single_window = single_window
        if label is None:
            label = "Project"
        self.__project_label = label
        # If this is a pop up, we should check if the original widget already displays orbitals etc. and load
        # them if possible to this window, too.
        if single_window:
            if self.autocas_settings.molecule_xyz_file:
                self.signal_handler.load_xyz_file_signal.emit(self.autocas_settings.molecule_xyz_file)
            if self.autocas_settings.molcas_orbital_file:
                self.signal_handler.load_molden_file_signal.emit(self.autocas_settings.molcas_orbital_file)

    def contextMenuEvent(self, event):
        """
        Override QWidget function to add a custom context menu.
        """
        if not self.__single_window:
            self.menu_function(event)

    def menu_function(self, event):
        menu = QMenu()
        expand_action = menu.addAction('Expand')
        action = menu.exec_(event.globalPos())  # type: ignore
        if action == expand_action:
            self.__show_project_in_new_window()

    def __show_project_in_new_window(self):
        self.__expanded_view = AutocasProjectWidget(None, self.signal_handler, self.autocas_settings,
                                                    self.__project_label, True)
        self.__expanded_view.setWindowTitle(self.__project_label)
        self.__expanded_view.resize(600, 500)
        self.__expanded_view.show()



__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

import sys
from typing import TYPE_CHECKING, Any

from PySide2.QtCore import QObject  # , QProcess

if TYPE_CHECKING:
    Signal = Any
else:
    pass
from PySide2.QtCore import Signal
from PySide2.QtWidgets import QTextEdit, QVBoxLayout, QWidget


class OutputWidget(QWidget):
    class EmittingStream(QObject):
        textWritten = Signal(str)

        def __init__(self):
            QObject.__init__(self)

        def write(self, text):
            self.textWritten.emit(str(text))

    def __init__(self, parent: QWidget):
        QWidget.__init__(self, parent)
        self.output = QTextEdit()
        self.__layout = QVBoxLayout()
        self.__layout.addWidget(self.output)
        # self.process = QProcess(self)
        # self.process.readyRead.connect(self.writeOutput)

        # sys.stdout = self.EmittingStream()
        # sys.stdout.textWritten.connect(self.writeOutput)

        self.setLayout(self.__layout)

    def __del__(self):
        sys.stdout = sys.__stdout__

    def writeOutput(self, text):
        cursor = self.output.textCursor()
        cursor.movePosition(cursor.End)
        # cursor.insertText(str(text))
        cursor.insertText(str(text))
        self.output.ensureCursorVisible()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""
"""
Tools for writing a molecule to a file.
"""

from pathlib import Path
from vtk import vtkMolecule
from typing import Union
import os

from scine_heron.molecule.utils.molecule_utils import molecule_to_atom_collection, molecule_to_bond_order_collection


def write_molecule_to_file(molecule: vtkMolecule, file_name: Union[Path, str]) -> None:
    """
    This method writes molecule in specified format as supported by SCINE Utilities.
    """
    import scine_utilities as utils

    _, extension = os.path.splitext(file_name)
    if extension in [".mol", ".pdb"]:
        utils.io.write_topology(str(file_name), molecule_to_atom_collection(molecule),
                                molecule_to_bond_order_collection(molecule))
    else:
        utils.io.write(str(file_name), molecule_to_atom_collection(molecule))


def write_trajectory_to_file(trajectory, file_name: Union[Path, str]) -> None:
    """
    This method writes a trajectory in specified format as supported by SCINE Utilities.
    """
    import scine_utilities as utils

    _, extension = os.path.splitext(file_name)
    if extension == ".bin":
        file_format = utils.io.TrajectoryFormat.Binary
    elif extension == ".xyz":
        file_format = utils.io.TrajectoryFormat.Xyz
    else:
        raise RuntimeError(f"Unsupported file extension {extension}, only support 'bin' and 'xyz'.")

    utils.io.write_trajectory(file_format, str(file_name), trajectory)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

import signal
from time import sleep
from typing import Dict, Optional, Any, List, TYPE_CHECKING, Tuple, Callable
from threading import Thread

from PySide2.QtGui import QCloseEvent
from PySide2.QtWidgets import (
    QLabel,
    QPushButton,
    QWidget
)
from PySide2.QtCore import QObject, QThread
if TYPE_CHECKING:
    Signal = Any
else:
    from PySide2.QtCore import Signal

import scine_utilities as su
import scine_readuct as readuct  # noqa  pylint: disable=unused-import  # used via eval

from scine_heron.containers.buttons import TextPushButton
from scine_heron.settings.class_options_widget import ClassOptionsWidget
from scine_heron.settings.docstring_parser import DocStringParser
from scine_heron.utilities import write_error_message, thread_safe_error


from scine_heron.containers.start_stop_widget import StartStopWidget


class TaskWidget(StartStopWidget):
    """
    Widget for any ReaDuct Task
    """

    finished_task = Signal()
    """
    emitted when the task is finished
    """
    update_systems = Signal(list, list, dict)
    """
    update_systems
        list of input names of the task they got created from
        list of names to be updated
        a dictionary with the names as keys and calculators as values
    """

    def __init__(
        self,
        parent: QWidget,
        inputs: List[str],
        task_name: str,
        widget_title: str,
        task_settings: Dict[str, Any],
        settings_suggestions: Optional[List[str]] = None
    ) -> None:
        """
        Construct a task widgets with all the required details to execute it.
        Task settings can still be modified later

        Parameters
        ----------
        parent : QWidget
            The parent of the widget, has to actually be a ReaductTab widget,
            but cannot be specified due to circular imports
        inputs : List[str]
            The names of the input systems
        task_name : str
            The name of the task
        widget_title : str
            The title of the widget
        task_settings : Dict[str, Any]
            The settings of the task
        settings_suggestions : Optional[List[str]], optional
            An optional list of setting names that are suggested, when the user wants to add
            a task setting, by default None
        """
        super().__init__(parent)
        self._parent = parent
        self._result: Optional[Tuple[Dict[str, su.core.Calculator], bool]] = None
        self.inputs = inputs
        self._task_name = task_name
        self._task_settings = task_settings
        self._settings_suggestions = settings_suggestions
        outputs = self._task_settings.get("output")
        self.outputs: List[str] = self.inputs if outputs is None else outputs

        self.__task_is_joining = False

        # Create layout and add widgets
        self.name = widget_title
        self._layout.addWidget(QLabel(widget_title))

        self.button_settings = QPushButton("Settings")
        self.button_delete = QPushButton("Delete")
        self.button_start_stop = TextPushButton("Start", self.start_stop)
        self._layout.addWidget(self.button_start_stop)
        self._add_settings_and_delete_buttons()

        self._run_thread: Optional[RunThread] = None
        self._join_thread: Optional[Thread] = None
        self._finish_check: Optional[CheckForFinishedTask] = None
        self.finished_task.connect(lambda: self.join(force_join=True))
        self.finished_task.connect(lambda: self.__switch_status(False))

        self.init_arguments += [self.inputs, self._task_name, self._task_settings]

        self.setMaximumWidth(300)
        self.setLayout(self._layout)

    def _stop_finish_check(self):
        """
        Stop our sub thread that checks on our task if it is finished.
        """
        if self._finish_check is not None:
            if self._finish_check.is_running():
                self._finish_check.stop()
            self._finish_check.wait()

    def is_working(self) -> bool:
        """
        Return if the task we are wrapping is currently running.

        Returns
        -------
        bool
            True if the task is running, False otherwise
        """
        return self._run_thread is not None and self._run_thread.isRunning()

    def _stop(self) -> None:
        """
        Stop our task.
        """
        if self._run_thread is not None:
            self._run_thread.terminate()
            self.change_color()

    def _run(self, message_container: Optional[List[str]] = None) -> None:
        """
        Run our task. The argument controls if we run it in blocking or non-blocking mode.

        Parameters
        ----------
        message_container : Optional[List[str]], optional
            An optional list in which encountered error messages are stored, this ensures thread safety.
            It therefore also controls if the task is run non-blocking (empty list) or blocking (None),
            by default None
        """
        run_method = eval(f"readuct.run_{self._task_name.lower()}_task")  # pylint: disable=eval-used
        calculators = self._parent.get_systems()
        self.change_color(self.orange)
        if message_container is None:
            # non-blocking run
            self._run_thread = RunThread(self, target=run_method, args=(calculators, self.inputs),
                                         kwargs=self._task_settings)
            self._run_thread.error_signal.connect(  # pylint: disable=no-member
                write_error_message
            )
            self._run_thread.start()
            self._finish_check = CheckForFinishedTask(self, self.finished_task)
            self._finish_check.start()
        else:
            # blocking run
            try:
                self.__switch_status(True)
                self._result = run_method(calculators, self.inputs, **self._task_settings)
                self.finished_task.emit()
            except BaseException as e:
                message_container.append(str(e))
                self._result = ({}, False)
                self.change_color(self.red)
                self.__switch_status(False)

    def closeEvent(self, event: QCloseEvent) -> None:
        """
        Make sure we stop and join all our sub threads

        Parameters
        ----------
        event : QCloseEvent
            The close event
        """
        if self._join_thread is not None:
            self._join_thread.join()
        if self._run_thread is not None:
            # if self._run_thread.is_alive():
            if self._run_thread.isRunning():
                self._stop()
            self._run_thread.wait()
        self._stop_finish_check()
        super().closeEvent(event)

    def set_docstring_dict(self, doc_string_parser: DocStringParser) -> None:
        """
        Set the docstring dictionary for our task settings.
        Currently not supported because no clear API on ReaDuct's side.

        Parameters
        ----------
        doc_string_parser : DocStringParser
            The parser that could parse doc from Python code.
        """
        self._docstring_dict = {}

    def stop_class_if_working(self) -> None:
        """
        Stops the class if we are working, but does not do anything if we are not working
        """
        if self.is_working():
            self._stop()

    def __switch_status(self, status: bool):
        """
        Changes our appearance and settings access based on the changed status.

        Parameters
        ----------
        status : bool
            The new status
        """
        if status:
            # we are starting
            self.button_start_stop.setText("Stop")
            if self.button_settings is not None:
                self.button_settings.setEnabled(False)
        else:
            # we have stopped
            self.button_start_stop.setText("Start")
            if self.button_settings is not None:
                self.button_settings.setEnabled(True)

    def get_result(self) -> Tuple[Dict[str, su.core.Calculator], bool]:
        """
        Returns the results of the executed task just like a direct ReaDuct call.
        Returns an empty dictionary and False, if the task is not finsihed yet.

        Returns
        -------
        Tuple[Dict[str, su.core.Calculator], bool]
            The results as tuple with the first argument being the systems map containing all calculators
            and the second being a boolean signalling if the task was successful or not
        """
        if self._result is None:
            return {}, False
        return self._result

    def start_stop(self, start_all: bool = False, stop_all: bool = False,
                   message_container: Optional[List[str]] = None) -> None:
        """
        Starts or stops the widget, i.e. the wrapping ReaDuct task.
        Optional arguments alter the behavior based on the fact that this is not the only task being
        started or stopped.

        Parameters
        ----------
        start_all : bool, optional
            If all tasks in the parent container are started, however blocking vs. non-blocking behavior
            is governed by the passed message_container, by default False
        stop_all : bool, optional
            If all tasks in the parent container are stopped at once, by default False
        message_container : Optional[List[str]], optional
            An optional list in which encountered error messages are stored, this ensures thread safety.
            It therefore also controls if the task is run non-blocking (empty list) or blocking (None),
            by default None
        """
        if start_all and stop_all:
            thread_safe_error("Internal Error", message_container)
            return
        if start_all and not self.is_working():
            self.__switch_status(True)
            self._run(message_container)
        elif stop_all and self.is_working():
            if self.__task_is_joining:
                # expect separate join() call from caller of this method with stop_all=True
                return
            self._stop()
        elif not start_all and self.is_working():
            if self.__task_is_joining:
                thread_safe_error("Still waiting for task to finish", message_container)
                return
            self._stop()
            # do join with thread here
            self._join_thread = Thread(target=self.join)
            self._join_thread.start()
        elif not stop_all and not self.is_working():
            self._run(message_container)
            self.__switch_status(True)

    def join(self, force_join: bool = False) -> Tuple[Dict[str, su.core.Calculator], bool]:
        """
        Joins the sub thread and gather the result of the task to save it in this wrapping class
        and make it obtainable through the getter and also return the result on this method.

        Parameters
        ----------
        force_join : bool, optional
            no effect here, by default False

        Returns
        -------
        Tuple[Dict[str, su.core.Calculator], bool]
            The results as tuple with the first argument being the systems map containing all calculators
            and the second being a boolean signalling if the task was successful or not
        """
        if not self.is_working() and not force_join:
            # no join required if not working
            return self.get_result()
        if self._run_thread is None:
            return self.get_result()
        if self.__task_is_joining:
            while self.__task_is_joining:
                # in case joining was initiated before and is in progress, we let the current join finish
                # and don't run another join, but this method is expected to only return once it was joined,
                # so, we imitate join behavior by stalling here
                sleep(0.1)
            # join is concluded, simply return
            return self.get_result()
        self.__task_is_joining = True
        if self.is_working():
            self._stop()
        try:
            self._run_thread.wait()
        except BaseException:
            pass
        self._result = self._run_thread.result
        assert self._result is not None
        self._stop_finish_check()
        if not force_join or self.is_working():
            # change if we don't force join, or if we are force joining a running task
            self.__switch_status(False)
        systems, success = self._result
        if success or not self._task_settings.get("stop_on_error", True):
            # handle via signal to be thread safe
            self.update_systems.emit(self.inputs, self.outputs, systems)
        color = self.green if success else self.red
        self.change_color(color)
        self.__task_is_joining = False
        self._run_thread = None
        return self._result

    def _show_settings(self) -> None:
        """
        Opens the task settings as a pop-up, which allows the user to modify the task settings.
        """
        setting_widget = ClassOptionsWidget(
            options=self._task_settings, docstring=self._docstring_dict, parent=self,
            add_close_button=True, allow_removal=True, allow_additions=True,
            addition_suggestions=self._settings_suggestions,
            keys_excluded_from_io=['outputs']  # makes sure we don't overwrite the auto-generated outputs
        )
        setting_widget.exec_()

    def get_names(self) -> Tuple[str, List[str], List[str]]:
        return self._task_name, self.inputs, self.outputs

    def get_settings(self) -> Dict[str, Any]:
        return self._task_settings


class CheckForFinishedTask(QThread):
    """
    A thread that receives a TaskWidget parent, continuously checks if its task is finished,
    and if so sends out a signal and ends itself.
    """

    def __init__(self, parent: TaskWidget, signal_to_send: Signal):
        """
        Constructs the parent with the task it should check on.

        Parameters
        ----------
        parent : TaskWidget
            The widget that runs a ReaDuct task
        signal_to_send : Signal
            The signal that we should emit when the task is finished
        """
        super().__init__(parent)
        self._parent = parent
        self._signal = signal_to_send
        self._stop = False
        self._sleep_time = 1.0

    def stop(self):
        """
        Stops the thread prematurely
        """
        self._stop = True

    def is_running(self) -> bool:
        """
        If we are still running

        Returns
        -------
        bool
            True when running
        """
        return not self._stop

    def run(self):
        """
        Starts observing the task and sends out a signal when it is finished.
        """
        while not self._stop:
            sleep(self._sleep_time)
            if not self._parent.is_working():
                self._signal.emit()
                break
        self.exit(0)


class RunThread(QThread):
    """
    A thread that runs the ReaDuct task and stores the result in the thread object.
    """

    error_signal = Signal(str)
    """
    A signal to transfer error messages
    """

    def __init__(self, parent: Optional[QObject], target: Callable,  # pylint: disable=dangerous-default-value
                 args=(), kwargs={}) -> None:
        """
        Constructs the thread with the task to run as a function call.

        Parameters
        ----------
        parent : Optional[QObject]
            The parent object
        target : Callable
            The function to call
        args : tuple, optional
            The arguments to pass to the function, by default ()
        kwargs : dict, optional
            The keyword arguments to pass to the function, by default {}
        """
        super().__init__(parent)
        self.target = target
        self.args = args
        self.kwargs = kwargs
        self.result: Optional[Tuple[Dict[str, su.core.Calculator], bool]] = None

    def run(self):
        """
        Run the task and store the result in the thread object.
        """
        self.result = ({}, False)
        try:
            self.result = self.target(*self.args, **self.kwargs)
        except BaseException as e:
            self.error_signal.emit(str(e))
            self.result = ({}, False)
        self.exit(0)

    def wait(self, *args, **kwargs):

        def do_nothing(*_):
            pass

        signal.signal(signal.SIGABRT, do_nothing)
        try:
            super().wait(*args, **kwargs)
        except BaseException:
            pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""
"""
Provides the MolecularViewerWidget class.
"""

from pathlib import Path
from time import sleep
from typing import Any, Optional, List, TYPE_CHECKING
import PySide2
from PySide2.QtWidgets import (
    QWidget,
    QSplitter,
    QCheckBox,
    QVBoxLayout,
    QScrollArea,
    QScrollBar,
)
from PySide2.QtCore import QObject, QSize, QThread

import scine_utilities as utils

from scine_heron import get_core_tab
from scine_heron.calculators.create_calculator_widget import CreateCalculatorWidget
from scine_heron.molecule.molecule_widget import MoleculeWidget
from scine_heron.molecule.molecule_writer import write_molecule_to_file, write_trajectory_to_file
from scine_heron.edit_molecule.edit_molecule_widget import EditMoleculeWidget
from scine_heron.energy_profile.energy_profile_widget import EnergyProfileWidget
from scine_heron.settings.settings_status_manager import SettingsStatusManager
from scine_heron.settings.basic_settings_widget import BasicSettingsWidget
from scine_heron.status_manager import StatusManager
from scine_heron.haptic.haptic_client import HapticClient
from scine_heron.molecule.molecule_video import MoleculeVideoWithEnergyDialog
from scine_heron.utilities import thread_safe_error

if TYPE_CHECKING:
    Slot = Any
else:
    from PySide2.QtCore import Slot


class MolecularViewerWidget(QWidget):
    """
    Sets up a tab for molecular views.
    """
    settings_widget_layout = None

    def __init__(self, parent: QObject, haptic_client: HapticClient, method_family: Optional[str] = None,
                 program: Optional[str] = None):
        QWidget.__init__(self, parent)

        self.mol_widget: Optional[MoleculeWidget] = None
        self.__layout = QVBoxLayout()

        self.__haptic_client = haptic_client
        self.__trajectory_writer: Optional[TrajectoryWriter] = None

        self._settings_scroll_area = QScrollArea()
        self.vscrollbar = QScrollBar()
        self.create_calculator_widget = CreateCalculatorWidget(parent=self, method_family=method_family,
                                                               program=program)

        self.settings_status_manager = SettingsStatusManager(
            settings_change_signal=self.create_calculator_widget.settings_changed_signal
        )
        self.create_calculator_widget.emit_signal()
        self.visible = False
        self.basic_settings_widget = BasicSettingsWidget(self.settings_status_manager)
        self.basic_settings_widget.setMaximumWidth(300)

        # Edit Molecule - Add and remove atoms
        self.edit_molecule_widget = EditMoleculeWidget()
        self.edit_molecule_widget.setMaximumWidth(300)
        self.edit_molecule_widget.addAtom.connect(self.__add_atom_default_position)
        self.edit_molecule_widget.removeSelectedAtom.connect(self.__remove_selected_atoms)
        self.edit_molecule_widget.setVisible(False)

        self.hidden_button = QCheckBox("Edit Molecule Options")
        self.hidden_button.setFixedSize(QSize(320, 30))
        self.hidden_button.setChecked(False)
        self.hidden_button.toggled.connect(self.set_visible)  # pylint: disable=no-member

        self.energy_profile_widget = EnergyProfileWidget()

        self.continuously_update_positions = StatusManager(False)
        self.file_loaded = StatusManager(False)

        self.settings_widget: Optional[QWidget] = None
        self.create_molecule_widget(empty=True)

        self.settings_status_manager.opaque_signal.connect(
            self.__propagate_opacity
        )
        self.settings_status_manager.hamiltonian_changed.connect(
            self.deactivate_automatic_updates
        )

    def create_molecule_widget(
        self,
        file: Optional[Path] = None,
        atom_collection: Optional[Any] = None,
        bond_orders: Optional[Any] = None,
        empty: bool = False
    ) -> None:
        """
        Create molecule widget.
        """
        if file is None and atom_collection is None and not empty:
            self.settings_status_manager.error_message = "No Structure input provided."
            return
        if file is not None and atom_collection is not None:
            self.settings_status_manager.error_message = (
                "Cannot specify both AtomCollection and file to read."
            )
            return
        if file is not None and not file.exists():
            self.settings_status_manager.error_message = (
                "The input xyz file does not exist."
            )
            return
        if atom_collection is not None and len(atom_collection) == 0:
            self.settings_status_manager.error_message = (
                "The input AtomCollection is empty."
            )
            return

        self.deactivate_automatic_updates()
        self.__clean()

        # Update or add new widget
        if self.mol_widget is not None:
            self.mol_widget.update_molecule(
                file=file,
                atoms=atom_collection,
                bonds=bond_orders
            )
        else:
            self.mol_widget = MoleculeWidget(
                self.settings_status_manager,
                parent=self,
                file=file,
                atoms=atom_collection,
                bonds=bond_orders,
                haptic_client=self.__haptic_client,
                energy_status_manager=self.energy_profile_widget.energy_status_manager,
            )
            self.mol_widget.setMaximumHeight(2048)
            self.__fill_layout()

        if self.mol_widget.has_atoms():
            self.file_loaded.value = True
        else:
            self.file_loaded.value = False

        self.__trajectory_writer = TrajectoryWriter(self.mol_widget)
        self.__trajectory_writer.start()

    def update_molecule(
        self,
        file: Optional[Path] = None,
        atoms: Optional[object] = None,
        bonds: Optional[object] = None
    ):
        if self.mol_widget is not None:
            self.mol_widget.update_molecule(file=file, atoms=atoms, bonds=bonds)
            if self.mol_widget.has_atoms():
                self.file_loaded.value = True
            else:
                self.file_loaded.value = False

    def __fill_layout(self) -> None:
        vertical_splitter = QSplitter()
        vertical_splitter.setOrientation(PySide2.QtCore.Qt.Vertical)
        if self.mol_widget is None:
            vertical_splitter.addWidget(QWidget())
        else:
            vertical_splitter.addWidget(self.mol_widget)
        vertical_splitter.addWidget(self.energy_profile_widget)

        self.settings_widget = QWidget()

        MolecularViewerWidget.settings_widget_layout = QVBoxLayout()
        MolecularViewerWidget.settings_widget_layout.addWidget(self.basic_settings_widget)
        MolecularViewerWidget.settings_widget_layout.addWidget(self.create_calculator_widget)
        MolecularViewerWidget.settings_widget_layout.addWidget(self.hidden_button)
        MolecularViewerWidget.settings_widget_layout.addWidget(self.edit_molecule_widget)

        assert self.settings_widget_layout is not None
        self.settings_widget.setLayout(self.settings_widget_layout)

        self._settings_scroll_area.setWidget(self.settings_widget)
        self._settings_scroll_area.setWidgetResizable(True)
        self._settings_scroll_area.setHorizontalScrollBarPolicy(PySide2.QtCore.Qt.ScrollBarAlwaysOff)
        self.vscrollbar = self._settings_scroll_area.verticalScrollBar()
        self.vscrollbar.rangeChanged.connect(self.scroll)

        horizontal_splitter = QSplitter()
        horizontal_splitter.addWidget(vertical_splitter)
        horizontal_splitter.addWidget(self._settings_scroll_area)
        horizontal_splitter.setSizes([500, 300])

        self.__layout.addWidget(horizontal_splitter)
        self.setLayout(self.__layout)
        self.show()
        self.set_visible()

    def set_visible(self) -> None:
        self.visible = self.hidden_button.isChecked()
        self.edit_molecule_widget.setVisible(self.visible)
        if self.settings_widget is not None:
            self.settings_widget.updateGeometry()
        self.updateGeometry()

    @Slot(int, int)
    def scroll(self, minimum, maximum):
        value = maximum if self.visible else minimum
        self.vscrollbar.setValue(value)

    def save_molecule(self, file: Path) -> None:
        """
        Save molecule in a xyz file.
        """
        if self.mol_widget is not None:
            write_molecule_to_file(
                self.mol_widget.provide_data(), file
            )

    def show_history(self):
        if self.__layout.count() > 0 and self.__trajectory_writer is not None:
            trajectory = self.__trajectory_writer.get_trajectory()
            MoleculeVideoWithEnergyDialog(self, trajectory)

    def save_trajectory(self, file: Path) -> None:
        """
        Save trajectory in a xyz file.
        """
        if self.__layout.count() > 0 and self.__trajectory_writer is not None:
            traj = self.__trajectory_writer.get_trajectory()
            write_trajectory_to_file(traj, file)

    def revert_frames(self, n_frames: int = 25) -> None:
        """
        Reverts last n frames
        """
        if self.__layout.count() > 0 and self.__trajectory_writer is not None and self.mol_widget is not None:
            self.__trajectory_writer.pause()
            traj = self.__trajectory_writer.get_trajectory()
            n_back = min(n_frames, len(traj) - 1)
            wanted_frame = traj[-n_back]
            atoms = utils.AtomCollection(traj.elements, wanted_frame)
            # delete all frames after the wanted one
            # slices not supported in python bindings of delitem of trajectory when writing this
            for _ in range(n_back):
                del traj[-1]
            self.mol_widget.update_molecule(atoms=atoms)
            self.__trajectory_writer.unpause()

    def stop_trajectory_writing(self):
        if self.__trajectory_writer is not None:
            self.__trajectory_writer.cancel()

    def configure_main_widget_updates(self, status: bool) -> None:
        """
        Sets the state of automatic updates in the main widget.
        """
        if self.mol_widget is not None:
            self.mol_widget.set_calc_gradient_in_loop(status)

    def toggle_automatic_updates(self) -> None:
        """
        Toggles the automatic update status and emits
        a signal if the status changed.
        """
        self.continuously_update_positions.value = (
            not self.continuously_update_positions.value
        )

    def deactivate_automatic_updates(self) -> None:
        """
        Sets the automatic updates to enabled/disabled and emits
        a signal if the status changed.
        """
        if not self.continuously_update_positions.value:
            return

        self.continuously_update_positions.value = False

    def __clean(self) -> None:
        """
        Delete old widget.
        """
        if self.__trajectory_writer is not None:
            self.__trajectory_writer.cancel()

    def __add_atom_default_position(self, *args, **kwargs) -> None:  # type: ignore[no-untyped-def]
        self.settings_status_manager.selected_molecular_orbital = None
        if self.mol_widget is None:
            self.create_molecule_widget(empty=True)
            self.file_loaded.value = False
        if self.mol_widget is not None:
            self.mol_widget.add_atom_default_position(*args, **kwargs)
            self.file_loaded.value = True

    def __remove_selected_atoms(self) -> None:
        if self.mol_widget is None:
            return
        self.settings_status_manager.selected_molecular_orbital = None
        self.mol_widget.remove_selected_atoms()
        if not self.mol_widget.has_atoms():
            self.file_loaded.value = False

    def __propagate_opacity(self, opaque_indices: List[int], exclude_mode: bool = False) -> None:
        if self.mol_widget is None:
            return
        self.mol_widget.make_indices_opaque(opaque_indices, exclude_mode)


class TrajectoryWriter(QThread):

    def __init__(self, parent: Optional[MoleculeWidget] = None):
        super(TrajectoryWriter, self).__init__(parent)
        self._was_canceled = False
        self._was_paused = False
        self._sleep_time = 0.1
        self._trajectory = utils.MolecularTrajectory(minimum_rmsd_for_addition=0.01)  # type: ignore
        if parent is not None:
            atoms = parent.get_atom_collection()
            self._trajectory.elements = atoms.elements
            self._trajectory.push_back(atoms.positions, energy=float('inf'))

    def get_trajectory(self):
        return self._trajectory

    def cancel(self):
        self._was_canceled = True
        sleep(2 * self._sleep_time)

    def pause(self):
        self._was_paused = True

    def unpause(self):
        self._was_paused = False

    def run(self):
        while not self._was_canceled:
            sleep(self._sleep_time)
            while self._was_paused:
                sleep(0.1 * self._sleep_time)
            atoms = self.parent().get_atom_collection()
            if atoms.size() == 0:
                continue
            energy = self.parent().get_energy()
            if atoms.elements != self._trajectory.elements:
                # restart if molecule was changed
                self._trajectory.clear()
                self._trajectory.elements = atoms.elements
            if energy is not None:
                self._trajectory.push_back(atoms.positions, energy=energy)
            else:
                self._trajectory.push_back(atoms.positions, energy=float('inf'))
        self.exit(0)


def get_mol_viewer_tab(want_atoms_there: bool, message_container: Optional[List[str]] = None) \
        -> Optional[MolecularViewerWidget]:
    """
    message_container avoids to directly write to status bar in case this function is called by a thread,
    list because str are copied by value
    """
    tab = get_core_tab('molecule_viewer')
    if tab is None or not isinstance(tab, MolecularViewerWidget) or tab.mol_widget is None:
        thread_safe_error("Could not find molecule viewer", message_container)
        return None
    if want_atoms_there and not tab.mol_widget.has_atoms():
        thread_safe_error("Molecule viewer does not have any molecule loaded", message_container)
        return None
    return tab



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

import os

import scine_heron
from scine_heron.resources import resource_path

from PySide2.QtCore import Qt
from PySide2.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
)
from PySide2.QtSvg import QSvgWidget


class AboutDialog(QWidget):
    def __init__(
            self, parent: QWidget,
            window_title: str = "About Heron",
    ) -> None:
        super(AboutDialog, self).__init__(parent)
        self.setWindowTitle(window_title)

        # Class members for widgets
        self.__layout = QVBoxLayout()
        self.__logo_box = QWidget()
        self.__logo_box.setFixedSize(450, 250)
        self.__logo_box_layout = QHBoxLayout()
        self.__logo_box.setLayout(self.__logo_box_layout)
        heron_logo_path = os.path.join(resource_path(), 'heron_logo.svg')
        scine_logo_path = os.path.join(resource_path(), 'scine_logo.svg')
        self.__scine_logo = QSvgWidget(self)
        self.__scine_logo.load(scine_logo_path)
        self.__scine_logo.setFixedSize(200, 200)
        self.__logo_box_layout.addWidget(self.__scine_logo)
        self.__heron_logo = QSvgWidget(self)
        self.__heron_logo.load(heron_logo_path)
        self.__heron_logo.setFixedSize(200, 200)
        self.__logo_box_layout.addWidget(self.__heron_logo)
        self.__layout.addWidget(self.__logo_box)
        version = scine_heron.__version__
        self.__layout.addWidget(QLabel('Version:'))
        self.__layout.addWidget(QLabel(f'''
            {version}
        '''))
        self.__scine_link = QLabel('''
        https://scine.ethz.ch
        ''')
        self.__scine_link.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.__layout.addWidget(QLabel('SCINE Website:'))
        self.__layout.addWidget(self.__scine_link)
        self.__layout.addWidget(QLabel('Heron GitHub:'))
        self.__github_link = QLabel('''
        https://gitub.com/qcscine/heron
        ''')
        self.__github_link.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.__layout.addWidget(self.__github_link)
        self.__layout.addWidget(QLabel('Description:'))
        self.__layout.addWidget(QLabel('''
        Heron is a graphical user interface for all SCINE software packages.
        It is mainly developed in the Reiher group at ETH Zurich.
        '''))
        self.__layout.addWidget(QLabel('License:'))
        self.__layout.addWidget(QLabel('''
        This code is licensed under the 3-clause BSD license. Copyright
        ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
        '''))

        # Set dialog layout
        self.setLayout(self.__layout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from copy import deepcopy
from typing import Any, Dict, Optional, TYPE_CHECKING, Tuple

from PySide2.QtCore import Qt
from PySide2.QtWidgets import (
    QWidget,
    QLabel,
    QLineEdit,
)
if TYPE_CHECKING:
    Signal = Any
else:
    from PySide2.QtCore import Signal

# ensures we can load a calculator
import scine_sparrow  # noqa # pylint: disable=unused-import
import scine_utilities as su

from scine_heron.calculators.calculator import ScineCalculatorWrapper
from scine_heron.containers.buttons import TextPushButton
from scine_heron.containers.layouts import HorizontalLayout, VerticalLayout
from scine_heron.settings.class_options_widget import ClassOptionsWidget
from scine_heron.utilities import write_error_message


class CreateCalculatorWidget(QWidget):

    settings_changed_signal = Signal(str, str, su.Settings)

    def __init__(self, parent: Optional[QWidget] = None, method_family: Optional[str] = None,
                 program: Optional[str] = None, atoms: Optional[su.AtomCollection] = None,
                 settings: Optional[Dict[str, Any]] = None) -> None:
        super().__init__(parent)

        # immediately load Sparrow calculator, so we have less complications, because we always have a calculator
        self._loaded_method_family = "PM6" if method_family is None else method_family
        self._loaded_program = "Sparrow" if program is None else program
        self._calc_wrapper = ScineCalculatorWrapper(self._loaded_method_family, self._loaded_program,
                                                    hessian_required=False, settings=settings, atoms=atoms)

        # doc
        self._docstring_dict: Optional[Dict[str, str]] = None

        # calculators require Method family + program
        # method family
        self.method_family_edit = QLineEdit(self._loaded_method_family)
        mf_layout = HorizontalLayout([QLabel("Method family:"), self.method_family_edit])

        # program
        self.program_edit = QLineEdit(self._loaded_program)
        program_layout = HorizontalLayout([QLabel("Program:"), self.program_edit])

        # buttons
        self._button_settings = TextPushButton("Settings", self._edit_settings)
        self._button_load = TextPushButton("Load\ncalculator", self._load_calculator)
        self._button_layout = HorizontalLayout([self._button_load, self._button_settings])

        # connect signals
        self.method_family_edit.returnPressed.connect(self.program_edit.setFocus)  # pylint: disable=no-member
        self.method_family_edit.returnPressed.connect(self.program_edit.selectAll)  # pylint: disable=no-member
        self.program_edit.returnPressed.connect(self._button_load.click)  # pylint: disable=no-member
        self.program_edit.returnPressed.connect(self._button_load.animateClick)  # pylint: disable=no-member

        self._can_be_edited = True

        # fill layout
        self._layout = VerticalLayout([QLabel("Specify calculator")])
        self._layout.setAlignment(Qt.AlignTop)
        self._layout.add_layouts([mf_layout, program_layout, self._button_layout])
        self.setLayout(self._layout)
        self.setMinimumHeight(200)
        self.emit_signal()

    def get_calculator_args(self) -> Tuple[str, str]:
        return self._loaded_method_family, self._loaded_program

    def get_structure(self) -> Optional[su.AtomCollection]:
        if self._calc_wrapper.prev_molecule_version is None:
            write_error_message("No structure loaded into calculator yet")
            return None
        return self._calc_wrapper.get_calculator().structure

    def get_calculator(self) -> su.core.Calculator:
        return self._calc_wrapper.get_calculator()

    def get_settings(self) -> su.Settings:
        return self._calc_wrapper.get_settings()

    def toggle_editable(self):
        if self._can_be_edited:
            self.method_family_edit.setReadOnly(True)
            self.program_edit.setReadOnly(True)
            self._button_settings.setEnabled(False)
            self._button_load.setEnabled(False)
            self._can_be_edited = False
        else:
            self.method_family_edit.setReadOnly(False)
            self.program_edit.setReadOnly(False)
            self._button_settings.setEnabled(True)
            self._button_load.setEnabled(True)
            self._can_be_edited = True

    def update_settings(self, new_settings: Dict[str, Any]) -> None:
        result = self._calc_wrapper.update_settings(new_settings)
        if result:
            self.emit_signal()
        else:
            write_error_message("Failed to modify calculator settings")

    def _edit_settings(self) -> None:
        d_settings = deepcopy(self._calc_wrapper.get_settings())
        setting_widget = ClassOptionsWidget(
            d_settings, self._docstring_dict, parent=self, add_close_button=True, allow_removal=False
        )
        setting_widget.exec_()
        if self._calc_wrapper.update_settings(d_settings.as_dict()):
            self.emit_signal()

    def _load_calculator(self) -> None:
        method_family = self.method_family_edit.text()
        program = self.program_edit.text()
        atoms = self._calc_wrapper.get_structure()
        if self._calc_wrapper.load_calculator(method_family, program, atoms=atoms):
            self._loaded_method_family = method_family
            self._loaded_program = program
            self.emit_signal()
            self._docstring_dict = self._calc_wrapper.get_docstring_dict()

    def emit_signal(self) -> None:
        self.settings_changed_signal.emit(self._loaded_method_family, self._loaded_program,
                                          self._calc_wrapper.get_settings())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import Optional, List

from PySide2.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    QTreeWidget,
    QTreeWidgetItem,
    QFileDialog,
    QLineEdit
)
from PySide2.QtCore import QObject

from scine_heron.reaction_templates.reaction_template_storage import ReactionTemplateStorage
from scine_heron.utilities import (
    color_axis,
    color_figure,
    get_font,
    get_primary_line_color,
    get_secondary_line_color,
    get_color_by_key,
)

from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg
from matplotlib.figure import Figure
from networkx import Graph, spring_layout
import numpy as np


class RTGraphCanvas(FigureCanvasQTAgg):
    def __init__(self, width=5, height=4):
        self.fig = Figure(figsize=(width, height))
        self.ax1 = self.fig.add_subplot(1, 1, 1)
        color_figure(self.fig)
        self.font = get_font()
        color_axis(self.ax1)
        self.ax1.axis('off')
        super(RTGraphCanvas, self).__init__(self.fig)

    def plot_graph(self, graph: Graph, mapping: dict) -> None:
        self.ax1.clear()

        fixed_pos = {
            0: (-0.05, 0.0),
            1: (+0.05, 0.0)
        }
        r_mol = 0.1
        n_lhs_mol = len(mapping['lhs']['mol'])
        tmp_space = np.linspace(-np.pi / 4, np.pi / 4, n_lhs_mol + 2)
        for i, mol in enumerate(mapping['lhs']['mol']):
            angle = tmp_space[i + 1]
            x = -r_mol * np.cos(angle) - 0.05
            y = -r_mol * np.sin(angle)
            fixed_pos[mol] = (x, y)
        n_rhs_mol = len(mapping['rhs']['mol'])
        tmp_space = np.linspace(-np.pi / 4, np.pi / 4, n_rhs_mol + 2)
        for i, mol in enumerate(mapping['rhs']['mol']):
            angle = tmp_space[i + 1]
            x = r_mol * np.cos(angle) + 0.05
            y = r_mol * np.sin(angle)
            fixed_pos[mol] = (x, y)
        r_frag = 0.2
        lhs_frag_bounds = np.linspace(-np.pi / 4, np.pi / 4, n_lhs_mol + 1)
        for i, mol in enumerate(mapping['lhs']['frag']):
            tmp_space = np.linspace(lhs_frag_bounds[i], lhs_frag_bounds[i + 1], len(mol) + 2)
            for j, frag in enumerate(mol):
                angle = tmp_space[j + 1]
                x = -r_frag * np.cos(angle) - 0.05
                y = -r_frag * np.sin(angle)
                fixed_pos[frag] = (x, y)
        rhs_frag_bounds = np.linspace(-np.pi / 4, np.pi / 4, n_rhs_mol + 1)
        for i, mol in enumerate(mapping['rhs']['frag']):
            tmp_space = np.linspace(rhs_frag_bounds[i], rhs_frag_bounds[i + 1], len(mol) + 2)
            for j, frag in enumerate(mol):
                angle = tmp_space[j + 1]
                x = r_frag * np.cos(angle) + 0.05
                y = r_frag * np.sin(angle)
                fixed_pos[frag] = (x, y)
        r_atom = 0.35
        for i, mol in enumerate(mapping['lhs']['atoms']):
            atom_bounds = np.linspace(lhs_frag_bounds[i], lhs_frag_bounds[i + 1], len(mol) + 1)
            for j, frag in enumerate(mol):
                tmp_space = np.linspace(atom_bounds[j], atom_bounds[j + 1], len(frag) + 2)
                for k, atom in enumerate(frag):
                    angle = tmp_space[k + 1]
                    x = -r_atom * np.cos(angle) - 0.05
                    y = -r_atom * np.sin(angle)
                    fixed_pos[atom] = (x, y)
        for i, mol in enumerate(mapping['rhs']['atoms']):
            atom_bounds = np.linspace(rhs_frag_bounds[i], rhs_frag_bounds[i + 1], len(mol) + 1)
            for j, frag in enumerate(mol):
                tmp_space = np.linspace(atom_bounds[j], atom_bounds[j + 1], len(frag) + 2)
                for k, atom in enumerate(frag):
                    angle = tmp_space[k + 1]
                    x = r_atom * np.cos(angle) + 0.05
                    y = r_atom * np.sin(angle)
                    fixed_pos[atom] = (x, y)
        pos = spring_layout(graph, dim=2, seed=42, k=0.1, pos=fixed_pos, fixed=fixed_pos.keys())

        for u, v in graph.edges():
            w = graph.get_edge_data(u, v)['weight']
            if w < 5 or w == 8:
                color = get_secondary_line_color()
            elif w in [6, 7]:
                color = get_color_by_key('reactionColor')
            else:
                color = get_primary_line_color()
            linestyle = 'solid'
            if w in [6, 8]:
                linestyle = ':'
            self.ax1.plot(*np.array([(pos[u], pos[v]), ]).T, linestyle=linestyle, color=color, zorder=0)
        # edge_xyz = np.array([(pos[u], pos[v]) for u, v in graph.edges()])

        # # Plot the edges
        # for vizedge in edge_xyz:
        #     self.ax1.plot(*vizedge.T, color=get_primary_line_color(), zorder=0)

        # Plot the nodes, one type at a time
        reaction = np.array([pos[0], pos[1]])
        tmp = []
        for mol in (mapping['lhs']['mol'] + mapping['rhs']['mol']):
            tmp.append(pos[mol])
        molecules = np.array(tmp)
        tmp = []
        for mol in (mapping['lhs']['frag'] + mapping['rhs']['frag']):
            for frag in mol:
                tmp.append(pos[frag])
        fragments = np.array(tmp)
        tmp = []
        atom_indices = []
        for mol in (mapping['lhs']['atoms'] + mapping['rhs']['atoms']):
            for frag in mol:
                for atom in frag:
                    tmp.append(pos[atom])
                    atom_indices.append(atom)
        atoms = np.array(tmp)
        self.ax1.scatter(*reaction.T, s=300, c=get_color_by_key('reactionColor'), zorder=1)
        for p in reaction:
            self.ax1.text(
                p[0], p[1], 'R',
                horizontalalignment='center', verticalalignment='center',
                size='medium', color=get_color_by_key('primaryTextColor'),
            )
        self.ax1.scatter(*molecules.T, s=300, c=get_color_by_key('compoundColor'), zorder=1)
        for p in molecules:
            self.ax1.text(
                p[0], p[1], 'M',
                horizontalalignment='center', verticalalignment='center',
                size='medium', color=get_color_by_key('primaryTextColor'),
            )
        self.ax1.scatter(*fragments.T, s=300, c=get_color_by_key('structureColor'), zorder=1)
        for p in fragments:
            self.ax1.text(
                p[0], p[1], 'F',
                horizontalalignment='center', verticalalignment='center',
                size='medium', color=get_color_by_key('primaryTextColor'),
            )
        self.ax1.scatter(*atoms.T, s=400, c=get_secondary_line_color(), zorder=1)
        for i, p in zip(atom_indices, atoms):
            self.ax1.text(
                p[0], p[1], graph.nodes[i]['at'],
                horizontalalignment='center', verticalalignment='center',
                size='medium', color=get_color_by_key('primaryTextColor'),
            )
        self.ax1.axis('off')
        self.fig.canvas.draw()
        self.fig.canvas.flush_events()


class TemplateTreeWidget(QTreeWidget):
    def __init__(self, parent: QObject):
        QTreeWidget.__init__(self, parent)

    # def contextMenuEvent(self, event):
    #     self.currentItem().contextMenuEvent(event)


class TemplateTreeWidgetItem(QTreeWidgetItem):

    def __init__(self, parent: QObject, info: List[str]):
        QTreeWidgetItem.__init__(self, parent, info, type=QTreeWidgetItem.UserType)
        self.id_string_list = [info]
        self.item_type = info


class ReactionTemplateDialog(QWidget):
    def __init__(
            self, parent: QWidget,
            storage: ReactionTemplateStorage,
            window_title: str = "Reaction Template Storage",
    ) -> None:
        super(ReactionTemplateDialog, self).__init__(parent)
        self.setWindowTitle(window_title)

        # Class members for storage of data
        self.__storage = storage

        # Class members for widgets
        self.__layout = QVBoxLayout()
        self.__button_box = QWidget()
        self.__button_box_layout = QHBoxLayout()
        self.__button_clear = QPushButton("Clear Storage")
        self.__button_add = QPushButton("Add From File")
        self.__button_load = QPushButton("Load From File")
        self.__button_save = QPushButton("Save To File")

        self.__browsing_box = QWidget()
        self.__browsing_box_layout = QHBoxLayout()
        self.__template__tree = QTreeWidget(self)
        self.__template__tree.setColumnCount(1)
        self.__template__tree.setHeaderHidden(True)
        self.__template__tree.itemClicked.connect(self.__focus_graph)  # pylint: disable=no-member
        self.__template__tree.itemActivated.connect(self.__focus_graph)  # pylint: disable=no-member
        self.__graph_canvas = RTGraphCanvas()

        # Create layout and add widgets
        self.__button_box_layout.addWidget(self.__button_clear)
        self.__button_box_layout.addWidget(self.__button_add)
        self.__button_box_layout.addWidget(self.__button_load)
        self.__button_box_layout.addWidget(self.__button_save)

        self.__button_box.setLayout(self.__button_box_layout)
        self.__layout.addWidget(self.__button_box)

        self.__searchbar = QLineEdit()
        self.__searchbar.textChanged.connect(self.update_tree_view)  # pylint: disable=no-member
        self.__layout.addWidget(self.__searchbar)

        self.__browsing_box_layout.addWidget(self.__template__tree)
        self.__browsing_box_layout.addWidget(self.__graph_canvas)

        self.__browsing_box.setLayout(self.__browsing_box_layout)
        self.__layout.addWidget(self.__browsing_box)

        # Connect functions to buttons and widgets
        self.__button_clear.clicked.connect(self.__press_clear)  # pylint: disable=no-member
        self.__button_add.clicked.connect(self.__press_add)  # pylint: disable=no-member
        self.__button_load.clicked.connect(self.__press_load)  # pylint: disable=no-member
        self.__button_save.clicked.connect(self.__press_save)  # pylint: disable=no-member

        # Set dialog layout
        self.setLayout(self.__layout)
        self.update_tree_view()

    def update_tree_view(self):
        self.__template__tree.clear()
        self.__tree_items = []
        for rt_id in self.__storage.get_template_ids():
            search_string = self.__searchbar.text().strip()
            if search_string and search_string not in rt_id:
                continue
            base_item = TemplateTreeWidgetItem(self.__template__tree, [f"{rt_id}"])
            self.__tree_items.append(base_item)
        if self.__tree_items:
            self.__focus_graph(self.__tree_items[0], None)
        self.__template__tree.insertTopLevelItems(0, self.__tree_items)

    def __press_clear(self) -> None:
        self.__storage.clear()
        self.update_tree_view()

    def __get_reaction_template_database_file(self) -> Optional[str]:
        filename, _ = QFileDialog.getOpenFileName(
            self,
            self.tr("Open File"),  # type: ignore[arg-type]
            "",
            self.tr("Reaction Template Database (*rtdb.pickle.obj)"),  # type: ignore[arg-type]
        )
        if filename:
            return str(filename)
        return None

    def __press_add(self) -> None:
        filename = self.__get_reaction_template_database_file()
        if filename:
            self.__storage.add_database(filename)
            self.update_tree_view()

    def __press_load(self) -> None:
        filename = self.__get_reaction_template_database_file()
        if filename:
            self.__storage.load_database(filename)
            self.update_tree_view()

    def __press_save(self) -> None:
        filename, _ = QFileDialog.getSaveFileName(
            self,
            self.tr("Open File"),  # type: ignore[arg-type]
            "",
            self.tr("Reaction Template Database (*rtdb.pickle.obj)"),  # type: ignore[arg-type]
        )
        if filename:
            self.__storage.save_database(filename)

    def __draw_rt_graph(self, rt_id: str) -> None:
        template = self.__storage.get_template(rt_id)
        if template:
            graph, mapping = template.get_networkx_graph()
            self.__graph_canvas.plot_graph(graph, mapping)

    def __focus_graph(self, item: TemplateTreeWidgetItem, _):
        rt_id = item.id_string_list[0][0]
        self.__draw_rt_graph(rt_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""
"""
Provides the HapticInteractorStyle class.
"""

from typing import Dict, Any, Tuple, List
from vtk import (
    vtkActor,
    vtkRenderer,
    vtkMoleculeMapper,
)

from typing import Optional, Callable
from scine_heron.haptic.haptic_client import HapticClient
from scine_heron.haptic.haptic_pointer_data import HapticPointerData
from scine_heron.haptic.haptic_pointer_representation import HapticPointerRepresentation
from scine_heron.molecule.styles.mouse_interactor_style import (
    MouseInteractorStyle,
    InteractionMode,
)
from scine_heron.electronic_data.electronic_data_status_manager import (
    ElectronicDataStatusManager,
)

from vtkmodules.qt.QVTKRenderWindowInteractor import QVTKRenderWindowInteractor


class HapticInteractorStyle(MouseInteractorStyle):
    """
    An interactor style (based on "MouseInteractorStyle") that supports
    picking and moving particles with the haptic device.
    """

    def __init__(
        self,
        interactor: QVTKRenderWindowInteractor,
        renderer: vtkRenderer,
        mapper: vtkMoleculeMapper,
        haptic_client: Optional[HapticClient],
        actors_dict: Dict[str, vtkActor],
        selected_atom_callback: Callable[[Optional[List[int]]], None],
    ):
        super().__init__(
            interactor, renderer, mapper, actors_dict, selected_atom_callback,
        )

        self.__haptic_client = haptic_client
        self.__last_haptic_pos: Tuple[float, float, float] = (0, 0, 0)
        self.__add_haptic_pointer()

    def set_electronic_data_status_manager(
        self, electronic_data_status_manager: ElectronicDataStatusManager
    ) -> None:
        self._electronic_data_status_manager = electronic_data_status_manager

    def __add_haptic_pointer(self) -> None:
        if (
            self.__haptic_client is not None
            and self.__haptic_client.device_is_available
            and self.__haptic_client.callback is not None
        ):
            self.__haptic_pointer = HapticPointerData()
            self.__haptic_pointer_representation = HapticPointerRepresentation(
                self.__haptic_pointer
            )
            self._renderer.AddActor(self.__haptic_pointer_representation.actor)
            self.__haptic_client.callback.signals.first_button_down_signal.connect(
                self._handle_left_button_press
            )
            self.__haptic_client.callback.signals.first_button_up_signal.connect(
                self._handle_left_button_release
            )
            self.__haptic_client.callback.signals.second_button_down_signal.connect(
                self.__handle_right_button_press_from_haptic
            )
            self.__haptic_client.callback.signals.second_button_up_signal.connect(
                self.__handle_right_button_release_from_haptic
            )
            self.__haptic_client.callback.signals.move_signal.connect(
                self.__handle_haptic_move
            )
        else:
            return

    @property
    def haptic_pointer(self) -> HapticPointerData:
        return self.__haptic_pointer

    def __handle_right_button_press_from_haptic(self, index: int) -> None:
        """
        Sets the member haptic_picked_atom_id to the picked atom.
        """
        self._pressed_buttons.add("right_haptic")
        self._settings_status_manager.haptic_picked_atom_id = index

    def __handle_right_button_release_from_haptic(
        self, _1: Optional[Any] = None, _2: Optional[Any] = None
    ) -> None:
        """
        Resets the member haptic_picked_atom_id to None and update atom position in haptic_device.
        """
        self._pressed_buttons.remove("right_haptic")
        if (
            self.__haptic_client is not None
            and self._settings_status_manager.haptic_picked_atom_id is not None
        ):
            self.__haptic_client.update_atom(
                self._settings_status_manager.haptic_picked_atom_id,
                self._molecule.GetAtom(self._settings_status_manager.haptic_picked_atom_id),
            )
            self._selected_atom_callback([self._settings_status_manager.haptic_picked_atom_id])
        else:
            self._selected_atom_callback(None)

    def _rotate_camera(self, azimuth: Any, elevation: Any) -> None:
        super()._rotate_camera(azimuth, elevation)
        if self.__haptic_client is not None:
            self.__haptic_client.update_transform_matrix(
                self._renderer.GetActiveCamera(), azimuth, elevation
            )

    def _move_atoms_by_mouse(self) -> None:
        super()._move_atoms_by_mouse()
        if (
            self.__haptic_client is not None
            and self._settings_status_manager.mouse_picked_atom_ids is not None
        ):
            for id in self._settings_status_manager.mouse_picked_atom_ids:
                self.__haptic_client.update_atom(
                    id,
                    self._molecule.GetAtom(id),
                )

    def __handle_haptic_move(
        self, pos: Any, azimuth: float, elevation: float, zoom: float
    ) -> None:
        """
        Moves the picked atom to the location defined by the mouse position.
        """

        self.__haptic_pointer.update_pointer_position(pos)
        last_pos = self.__last_haptic_pos
        self.__last_haptic_pos = pos.x, pos.y, pos.z

        if self._settings_status_manager.haptic_picked_atom_id is not None:
            interaction_mode = self._interaction_mode(
                self._pressed_buttons, [self._settings_status_manager.haptic_picked_atom_id]
            )
        else:
            interaction_mode = self._interaction_mode(
                self._pressed_buttons, None
            )

        if interaction_mode == InteractionMode.ROTATE_CAMERA:
            self._rotate_camera(azimuth, elevation)
            self._renderer.GetActiveCamera().Zoom(zoom)
            self._render()
        elif interaction_mode == InteractionMode.MOVE_CAMERA:
            move_pos = (
                last_pos[0] - pos.x,
                last_pos[1] - pos.y,
                last_pos[2] - pos.z,
            )
            self._move_camera(move_pos)
        elif interaction_mode == InteractionMode.MOVE_ATOMS:
            self._settings_status_manager.selected_molecular_orbital = None
            self._settings_status_manager.number_of_molecular_orbital = None

            atom = self._molecule.GetAtom(self._settings_status_manager.haptic_picked_atom_id)
            old_pos = atom.GetPosition()
            atom.SetPosition(pos.x, pos.y, pos.z)
            distance = [pos.x - old_pos[0], pos.y - old_pos[1], pos.z - old_pos[2]]
            if (
                self._settings_status_manager.mouse_picked_atom_ids is not None
                and len(self._settings_status_manager.mouse_picked_atom_ids) > 1
                and (
                    self._settings_status_manager.haptic_picked_atom_id
                    in self._settings_status_manager.mouse_picked_atom_ids
                )
            ):
                for atom_id in self._settings_status_manager.mouse_picked_atom_ids:
                    if atom_id is not self._settings_status_manager.haptic_picked_atom_id:
                        atom = self._molecule.GetAtom(atom_id)
                        old_pos = atom.GetPosition()
                        atom.SetPosition(
                            old_pos[0] + distance[0],
                            old_pos[1] + distance[1],
                            old_pos[2] + distance[2],
                        )
            self._render()
        else:
            self._render()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""


from .io_toolbar import HeronToolBar

from PySide2.QtWidgets import (
    QStyle,
    QFileDialog,
)
from pathlib import Path
from typing import Optional, Any, TYPE_CHECKING
from PySide2.QtCore import QObject
if TYPE_CHECKING:
    Signal = Any
else:
    from PySide2.QtCore import Signal

from scine_heron import get_core_tab
from scine_heron.readuct.readuct_tab import ReaductTab
from scine_heron.utilities import write_error_message, module_available


class MOToolbar(HeronToolBar):
    """
    A toolbar with a load and a save button.
    """
    load_file_signal = Signal(Path)
    save_file_signal = Signal(Path)
    save_trajectory_signal = Signal(Path)
    back_signal = Signal()
    display_history = Signal()
    toggle_updates_signal = Signal()

    def __init__(self, parent: Optional[QObject] = None):
        super(MOToolbar, self).__init__(parent=parent)
        self.db_manager: Optional[Any] = None

        self.shortened_add_action(QStyle.SP_DialogOpenButton, "Load Molecule", "Ctrl+O", self.__load_file)
        self.__save_mol = self.shortened_add_action(
            "save_molecule.png", "Save Molecule", "Ctrl+S", self.__save_file)
        self.__save_trj = self.shortened_add_action("save_trajectory.png", "Save Trajectory",
                                                    "Ctrl+Shift+S", self.__show_history)
        self.__loop = self.shortened_add_action(QStyle.SP_MediaPlay, "Start updating positions with Sparrow", "Ctrl+F",
                                                self.__calc_gradient_in_loop)
        self.__loop.setCheckable(True)
        self.__revert = self.shortened_add_action(QStyle.SP_MediaSeekBackward, "Revert last frames",
                                                  "Ctrl+Z", self.__revert_frames)
        self.__readuct = self.shortened_add_action(QStyle.SP_ArrowForward, "Move to ReaDuct",
                                                   "Ctrl+R", self.__move_to_readuct)
        self.__readuct_available = module_available("scine_readuct")
        self.__loop.setEnabled(False)
        self.__save_mol.setEnabled(False)
        self.__save_trj.setEnabled(False)
        self.__revert.setEnabled(False)
        self.__readuct.setEnabled(False)
        self.__automatic_updates = False
        self.setMinimumWidth(3000)

    def reset_plot(self) -> None:
        from scine_heron.energy_profile.energy_profile_widget import (
            EnergyProfileWidget
        )
        ax = EnergyProfileWidget._EnergyProfileWidget__canvas._Canvas__line.axes
        ax.axis([-0.06, 0.06, -0.06, 0.06])
        EnergyProfileWidget._EnergyProfileWidget__canvas.draw()

    def __load_file(self) -> None:
        """
        Load molecule from file.
        """
        filename, _ = QFileDialog.getOpenFileName(
            self,
            self.tr("Open File"),  # type: ignore[arg-type]
            "",
            self.tr("Molecule (*.xyz *.mol *.pdb)"),  # type: ignore[arg-type]
        )

        if filename:
            self.load_file_signal.emit(Path(filename))
            self.reset_plot()

    def __save_file(self) -> None:
        """
        Save molecule to file.
        """
        filename, _ = QFileDialog.getSaveFileName(
            self,
            self.tr("Save File"),  # type: ignore[arg-type]
            "molecule.xyz",
            self.tr("Molecule (*.xyz *.mol *.pdb)"),  # type: ignore[arg-type]
        )

        if filename:
            self.save_file_signal.emit(Path(filename))

    def __show_history(self) -> None:
        self.display_history.emit()

    def __revert_frames(self) -> None:
        """
        Send signal to revert frames
        """
        self.back_signal.emit()

    def __calc_gradient_in_loop(self) -> None:
        """
        Start or stop calculating the gradient in an infinity loop.
        """
        self.__update_display()
        self.toggle_updates_signal.emit()

    def __update_display(self) -> None:
        """
        Updates the state of the `__loop` button.
        """
        self.__loop.setChecked(self.__automatic_updates)
        self.__loop.setIcon(
            self.style().standardIcon(QStyle.SP_MediaPause)
            if self.__automatic_updates
            else self.style().standardIcon(QStyle.SP_MediaPlay)
        )

    def __move_to_readuct(self) -> None:
        tab = get_core_tab('readuct')
        if tab is None or not isinstance(tab, ReaductTab):
            write_error_message("ReaDuct tab could not be localized")
            return
        tab.add_system_from_molecular_viewer()

    def show_updates_enabled(self, status: bool) -> None:
        """
        Display the provided automatic updates status.
        """
        if status == self.__automatic_updates:
            return

        self.__automatic_updates = status
        self.__update_display()

    def show_file_loaded(self, status: bool) -> None:
        """
        Adapt the display for a loaded file.
        """
        self.__loop.setEnabled(status)
        self.__save_mol.setEnabled(status)
        self.__save_trj.setEnabled(status)
        self.__revert.setEnabled(status)
        self.__readuct.setEnabled(status and self.__readuct_available)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from inspect import isfunction
from typing import Any, Optional
import importlib

from .dummy import AnyMatchingClass, NotMatchingClass, DummyClass


def importer(module: str, attr: Optional[str] = None, return_any: bool = False):
    """
    This function is used to import optional dependencies.

    Parameters
    ----------
    module : str
        The module to import given as 'scine_heron.dependencies.optional_import'.
    attr : str, optional
        The attribute to import from the module, e.g. 'importer'.
    return_any : bool, optional
        If True, the function will return AnyMatchingClass instead of NotMatchingClass if the dependency is not
        installed.

    Returns
    -------
    result : Any
    """
    try:
        if attr is not None:
            return getattr(importlib.import_module(module), attr)
        return importlib.import_module(module)
    except (ImportError, AttributeError):
        return AnyMatchingClass if return_any else NotMatchingClass


def is_imported(obj: Any) -> bool:
    """
    Checks if the given object is a dummy object or a proper instance, type, or function.

    Returns True if the object is not a dummy object, False otherwise.
    """
    if isfunction(obj):
        return True
    try:
        return not issubclass(obj, DummyClass)
    except TypeError:
        return not isinstance(obj, DummyClass)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""
"""
Functions that abstract sending and receiving data
between a server and a client.
The data is now serialized using json.
"""
import json
import struct
import typing
from socket import socket as Socket


def send_data(data: typing.Any, socket: Socket) -> None:
    """
    Serialise and send data through a socket.
    The size of the dataset is prepended to the real data,
    so that the receiver knows how large is the dataset.
    Retry until it is done.
    """
    serialized = json.dumps(data)
    raw_data = bytes(serialized, encoding="utf-8")
    sizeinfo = struct.pack("!i", len(raw_data))  # Sending an integer

    socket.sendall(sizeinfo)
    socket.sendall(raw_data)


def recv_data(connection: Socket) -> typing.Any:
    """
    Read data from a socket.
    It is expected that the first datum that comes out
    will be the size of the data sent through.
    """
    buf = bytearray()
    while len(buf) < 4:
        buf += connection.recv(4 - len(buf))
    (size_to_receive,) = struct.unpack("!i", buf)
    size_received = 0
    data_received = bytearray()
    while len(data_received) < size_to_receive:
        chunksize = min(4096, size_to_receive - size_received)
        data_received += connection.recv(chunksize)
    data_decoded = data_received.decode("utf-8")

    return json.loads(data_decoded)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from scine_autocas import Autocas
from scine_autocas.autocas_utils.molecule import Molecule
from scine_autocas.interfaces.molcas import Molcas

from scine_heron.autocas.autocas_settings import AutocasSettings
from scine_heron.autocas.signal_handler import SignalHandler

from PySide2.QtCore import QCoreApplication
translate = QCoreApplication.translate


class AutocasWrapper:
    def __init__(
        self, autocas_settings: AutocasSettings, signal_handler: SignalHandler
    ):
        # self.molecule = Molecule()
        # self.autocas = Autocas(self.molecule)
        # self.interface = Molcas(self.molecule)
        self.molecule = None
        self.autocas = None
        self.interface = None
        self.settings = autocas_settings
        self.signal_handler = signal_handler

        self.signal_handler.start_initial_orbital_calculation_signal.connect(
            self.generate_initial_orbitals
        )
        self.signal_handler.start_initial_dmrg_calculation_signal.connect(
            self.run_initial_dmrg
        )
        self.signal_handler.start_final_calculation_signal.connect(
            self.run_final_calculation
        )

    def check_settings(self):
        # print("--------------")
        # print(self.settings.molecule_xyz_file)
        if self.settings.molecule_xyz_file is None:
            # print("huhuhh")
            self.signal_handler.load_xyz_file_signal.emit("")

        if self.molecule is None or self.autocas is None or self.interface is None:
            # self.signal_handler.open_molecule_widget_signal.emit()
            self.signal_handler.load_xyz_file_signal.emit(self.settings.molecule_xyz_file)
            self.settings.interface_xyz_file = self.settings.molecule_xyz_file
            self.molecule = Molecule(self.settings.molecule_xyz_file)
            self.autocas = Autocas(self.molecule)
            self.interface = Molcas(self.molecule)
        else:
            pass

    def apply_settings(self):
        self.settings.interface_xyz_file = self.settings.molecule_xyz_file

        # Molecule
        self.autocas.molecule.charge = self.settings.molecule_charge
        self.autocas.molecule.spin_multiplicity = (
            self.settings.molecule_spin_multiplicity
        )
        self.autocas.molecule.double_d_shell = self.settings.molecule_double_d_shell
        # self.autocas.molecule.core_orbitals = self.settings.molecule_core_orbitals
        # self.autocas.molecule.valence_orbitals = self.settings.molecule_valence_orbitals
        # self.autocas.molecule.electrons = self.settings.molecule_electrons
        # self.autocas.molecule.occupation = self.settings.molecule_occupation
        # self.autocas.molecule.xyz_file = self.settings.molecule_xyz_file

        # AutoCAS
        self.autocas.plateau_values = self.settings.plateau_values
        self.autocas.threshold_step = self.settings.threshold_step
        # AutoCAS Diagnostics
        self.autocas.diagnostics.weak_correlation_threshold = (
            self.settings.weak_correlation_threshold
        )
        self.autocas.diagnostics.single_reference_threshold = (
            self.settings.weak_correlation_threshold
        )
        # AutoCAS Large Spaces
        self.autocas.large_spaces.max_orbitals = self.settings.large_spaces_max_orbitals

        # Interface
        self.interface.dump = self.settings.molcas_dump
        self.interface.project_name = self.settings.molcas_project_name

        # Interface Settings
        self.interface.settings.dmrg_bond_dimension = (
            self.settings.interface_dmrg_bond_dimension
        )
        self.interface.settings.dmrg_sweeps = self.settings.interface_dmrg_sweeps
        self.interface.settings.basis_set = self.settings.interface_basis_set
        self.interface.settings.method = self.settings.interface_method
        self.interface.settings.post_cas_method = (
            self.settings.interface_post_cas_method
        )
        self.interface.settings.work_dir = self.settings.molcas_work_dir
        self.interface.settings.xyz_file = self.settings.interface_xyz_file
        self.interface.settings.point_group = self.settings.molcas_point_group
        self.interface.settings.ipea = self.settings.molcas_ipea
        self.interface.settings.cholesky = self.settings.interface_cholesky
        self.interface.settings.uhf = self.settings.interface_uhf
        self.interface.settings.fiedler = self.settings.interface_fiedler
        self.interface.settings.n_excited_states = self.settings.interface_n_excited_states
        # self.interface.settings.spin_multiplicity = (
        #     self.settings.interface_spin_multiplicity
        # )
        # self.interface.settings.charge = self.settings.interface_charge
        # self.interface.settings.only_hf = self.settings.interface_only_hf

        # MOLCAS
        # self.interface.settings.ci_root_string = self.settings.molcas_ci_root_string
        # self.interface.settings.only_hf = self.settings.molcas_only_hf
        # self.interface.settings.orbital_localisation = (
        #     self.settings.molcas_orbital_localisation
        # )
        # self.interface.settings.localisation_space = (
        #     self.settings.molcas_localisation_space
        # )
        # self.interface.settings.localisation_method = (
        #     self.settings.molcas_localisation_method
        # )

        self.interface.orbital_file = self.settings.molcas_orbital_file

    def generate_initial_orbitals(self):
        self.check_settings()
        self.apply_settings()
        self.interface.calculate()

        (
            self.settings.result_initial_occupation,
            self.settings.result_initial_indices,
        ) = self.autocas.make_initial_active_space()
        self.settings.user_occupation = self.settings.result_initial_occupation
        self.settings.user_indices = self.settings.result_initial_indices
        molden_file = self.settings.get_orbital_file()

        self.signal_handler.open_molecule_widget_signal.emit()
        self.signal_handler.load_molden_file_signal.emit(molden_file)

    def run_initial_dmrg(self):
        self.check_settings()
        self.apply_settings()

        # pylint: disable-next=W0612
        (
            self.settings.result_initial_energy,
            self.settings.result_initial_s1,
            initial_s2,
            self.settings.result_initial_mutual_information,
        ) = self.interface.calculate(
            self.settings.user_occupation,
            self.settings.user_indices
        )
        # self.interface.calculate(
        #     self.settings.user_occupation,
        #     self.settings.user_indices
        # )

        self.signal_handler.update_entanglement_plot_signal.emit(
            self.settings.result_initial_s1,
            self.settings.result_initial_mutual_information,
            self.settings.user_indices
        )

        self.signal_handler.open_molecule_widget_signal.emit()
        order = [x + 1 for x in self.settings.user_indices]
        # print(order)
        self.signal_handler.update_entanglement_plot_signal.emit(
            self.settings.result_initial_s1, self.settings.result_initial_mutual_information, order)

        # self.parent().entanglement_diagram.update_plot(
        #    initial_s1, initial_mutual_information, initial_indices
        # )

        self.settings.result_final_occupation, self.settings.result_final_indices = self.autocas.get_active_space(
            self.settings.user_occupation, self.settings.result_initial_s1, force_cas=True
        )
        self.settings.user_occupation = self.settings.result_final_occupation
        self.settings.user_indices = self.settings.result_final_indices

        # self.signal_handler.update_entanglement_plot_signal.emit()

    def run_final_calculation(self):
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
__copyright__ = """ This code is licensed under the 3-clause BSD license.
Copyright ETH Zurich, Department of Chemistry and Applied Biosciences, Reiher Group.
See LICENSE.txt for details.
"""

from typing import Any, Optional, Dict, Set, TYPE_CHECKING, Tuple

from PySide2.QtWidgets import (
    QWidget,
    QMenu,
    QLineEdit,
    QLabel,
    QVBoxLayout,
    QDialog,
    QScrollArea,
)
from PySide2.QtCore import Qt, QObject
if TYPE_CHECKING:
    Signal = Any
else:
    from PySide2.QtCore import Signal

from scine_utilities import AtomCollection
from scine_utilities import core, io, settings_names

from scine_heron.calculators.create_calculator_widget import CreateCalculatorWidget
from scine_heron.calculators.calculator import CalculatorLoadingFailed
from scine_heron.containers.buttons import TextPushButton
from scine_heron.containers.layouts import HorizontalLayout, VerticalLayout
from scine_heron.io.text_box import yes_or_no_question
from scine_heron.io.file_browser_popup import get_load_file_name
from scine_heron.containers.collapsible_list import CollapsibleList, CollapsibleListItem
from scine_heron.molecule.molecule_widget import MoleculeWidget
from scine_heron.molecular_viewer import get_mol_viewer_tab
from scine_heron.settings.dict_option_widget import DictOptionWidget
from scine_heron.utilities import write_info_message


class CalculatorContainer(QWidget):
    """
    Container Widget to handle ReaDuct systems / Scine Calculators
    """
    default_name = "enumeration"
    new_system_added = Signal(str)

    def __init__(self, parent: Optional[QObject]) -> None:
        super().__init__(parent)
        self._layout = QVBoxLayout()
        self._layout.setAlignment(Qt.AlignTop)

        self._default_method_family = "PM6"
        self._default_program = "Any"
        self._general_settings: Dict[str, Any] = {}
        self._names: Set[str] = set()

        # calc name
        self.name_edit = QLineEdit(self.default_name)
        name_layout = HorizontalLayout([QLabel("Name:"), self.name_edit])
        self._layout.addLayout(name_layout)

        # buttons
        self.molecular_viewer_add_button = TextPushButton("Add from molecular viewer", self.add_from_viewer)
        file_add_button = TextPushButton("Add from file", self.add_from_file)
        settings_button = TextPushButton("Default Settings", self.show_settings_widget)
        self._layout.addLayout(HorizontalLayout([self.molecular_viewer_add_button, file_add_button, settings_button]))

        # tree
        self._layout.addWidget(QLabel("Existing Systems:"))
        self.tree = CalculatorListWidget(self)
        self._scroll_area = QScrollArea()
        self._scroll_area.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOn)
        self._scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self._scroll_area.setWidgetResizable(True)
        self._scroll_area.setWidget(self.tree)
        self._scroll_area.setMinimumHeight(350)
        self._layout.addWidget(self._scroll_area)

        # viewer
        self._layout.addStretch()
        self._mol_widget = MoleculeWidget(parent=self, disable_modification=True)
        self._layout.addWidget(self._mol_widget)

        self.setLayout(self._layout)

    def get_systems(self) -> Dict[str, core.Calculator]:
        systems = {}
        for item in self.tree:
            systems[item.name] = item.get_calculator()
        return systems

    def get_augmented_systems(self) -> Dict[str, Tuple[str, str, core.Calculator]]:
        systems = {}
        for item in self.tree:
            method_family, program = item.get_calculator_args()
            systems[item.name] = (method_family, program, item.get_calculator())
        return systems

    def focus(self):
        item = self.tree.current_item()
        self._scroll_area.ensureWidgetVisible(item)
        self._scroll_area.updateGeometry()
        if item is not None:
            structure = item.get_structure()
            if structure is not None:
                self._mol_widget.update_molecule(atoms=structure)
                self.updateGeometry()

    def show_settings_widget(self):
        frame = QDialog(self)
        method_family_edit = QLineEdit(self._default_method_family)
        mf_layout = HorizontalLayout([QLabel("Method family:"), method_family_edit])
        program_edit = QLineEdit(self._default_program)
        program_layout = HorizontalLayout([QLabel("Program:"), program_edit])
        method_family_edit.returnPressed.connect(program_edit.setFocus)  # pylint: disable=no-member
        method_family_edit.returnPressed.connect(program_edit.selectAll)  # pylint: disable=no-member
        suggestions = [s for s in dir(settings_names) if not s.startswith("_")]
        settings_widget = DictOptionWidget(options=self._general_settings, parent=frame,
                                           add_close_button=False, allow_additions=True,
                                           addition_suggestions=suggestions)
        settings_widget.setMinimumHeight(200)
        layout = VerticalLayout()
        layout.setAlignment(Qt.AlignTop)
        layout.add_layouts([mf_layout, program_layout])
        layout.add_widgets([
            QLabel("Settings for all new calculators:"),
            settings_widget,
            TextPushButton("Ok", frame.reject)
        ])
        frame.setLayout(layout)
        frame.exec_()
        self._default_method_family = method_family_edit.text()
        self._default_program = program_edit.text()
        self._general_settings = settings_widget.get_widget_data()

    def _determine_name(self) -> str:
        name_line = self.name_edit.text()
        if name_line != self.default_name:
            return name_line
        return str(len(self.tree))

    def add_item(self, method_family: str, program: str, atoms: AtomCollection, settings: Dict[str, Any],
                 name: Optional[str] = None):
        if name is None:
            name = self._determine_name()
        if name in self._names:
            replace_name = yes_or_no_question(self, f"System '{name}' does already exist. Do you want to replace it")
            if replace_name:
                write_info_message(f"Replacing {name}")
                item = self.tree.get_item_by_name(name)
                if item is None:
                    raise RuntimeError(f"Internal name management error with {name} and {self._names}")
                self.tree.remove_widget(item)
            else:
                write_info_message("No system was added")
                return
        else:
            self._names.add(name)
        try:
            content = CreateCalculatorWidget(method_family=method_family, program=program, atoms=atoms,
                                             settings=settings)
            new_item = CollapsibleCalculatorListItem(self.tree, name, content)
        except CalculatorLoadingFailed:
            self._names.remove(name)
        else:
            self.tree.add_widget(new_item)
            self.tree.set_current_item(new_item)
            self.focus()
            self.new_system_added.emit(name)

    def add_from_viewer(self) -> None:
        tab = get_mol_viewer_tab(want_atoms_there=True)
        if tab is None or tab.mol_widget is None:
            return
        mol_viewer_calculator_edit = tab.create_calculator_widget
        atoms = tab.mol_widget.get_atom_collection()  # ensures that we get the current atoms
        method_family = mol_viewer_calculator_edit.method_family_edit.text()
        program = mol_viewer_calculator_edit.program_edit.text()
        if method_family != self._default_method_family or program != self._default_program:
            write_info_message("Taking method family and program from molecular viewer")
        mol_viewer_settings = mol_viewer_calculator_edit.get_settings()
        if any(default_key in mol_viewer_settings for default_key in self._general_settings.keys()):
            write_info_message("Duplicate keys in given defaults and molecular viewer. Given defaults overrule")
        settings = {**mol_viewer_calculator_edit.get_settings(), **self._general_settings}  # type: ignore
        self.add_item(method_family=method_family, program=program, atoms=atoms, settings=settings)

    def add_from_file(self) -> None:
        filename = get_load_file_name(self, "Structure", ["xyz", "mol", "pdb"])
        if filename is None:
            return
        atoms, bonds = io.read(str(filename))
        self.add_item(self._default_method_family, self._default_program, atoms, self._general_settings)
        if not bonds.empty():
            self._mol_widget.set_bonds(bonds)


class CollapsibleCalculatorListItem(CollapsibleListItem):

    def __init__(self, parent: QWidget, name: str, content: CreateCalculatorWidget) -> None:  # for typing
        super().__init__(parent, name, content)
        self.content = content

    def get_calculator_args(self):
        return self.content.get_calculator_args()

    def get_calculator(self):
        return self.content.get_calculator()

    def get_structure(self):
        return self.content.get_structure()

    def contextMenuEvent(self, event):
        menu = QMenu()
        show_in_mol_view = menu.addAction('Move to Molecular Viewer')
        _ = menu.addAction('Close Menu')
        action = menu.exec_(event.globalPos())  # type: ignore
        if action == show_in_mol_view:
            tab = get_mol_viewer_tab(want_atoms_there=False)
            if tab is not None:
                atoms = self.content.get_structure()
                if atoms is not None:
                    tab.update_molecule(atoms=atoms)
                    tab.layout().replaceWidget(tab.create_calculator_widget, self.content)


class CalculatorListWidget(CollapsibleList):

    def __init__(self, parent: QObject):
        super().__init__(parent)
        self._widgets: Dict[str, CollapsibleCalculatorListItem] = {}
